from __future__ import print_function, division, absolute_import

from warnings import warn
import numpy as np
import numpy.linalg as npl

from scipy.stats import rankdata

from . import subsample_columns, grouped_path, default_lagrange_sequence
from ..affine import astransform, power_L
from ..smooth import glm, affine_smooth, sum as smooth_sum
from ..smooth.quadratic import quadratic_loss
from ..problems.simple import simple_problem
from ..identity_quadratic import identity_quadratic as iq
from ..atoms.weighted_atoms import l1norm as weighted_l1norm

class lasso_path(grouped_path):

    BIG = 1e12 # lagrange parameter for finding null solution

    def __init__(self, 
                 saturated_loss,
                 X, 
                 lasso_weights,
                 elastic_net_param=None,
                 alpha=1.  # elastic net mixing -- 1 is LASSO
                 ):

        self.saturated_loss = saturated_loss
        self.X = astransform(X)
        # the penalty parameters

        self.alpha = alpha
        self.penalty = weighted_l1norm(lasso_weights, lagrange=1)
        self.shape = self.penalty.shape
        self.group_shape = self.penalty.shape # each feature is its own group

        # elastic net part
        if elastic_net_param is None:
            elastic_net_param = np.ones(self.shape)
        self.elastic_net_param = elastic_net_param

        unpenalized = self.penalty.weights == 0
        penalized = self._penalized_vars = ~unpenalized
        self.solution = np.zeros(self.shape)
        self.ever_active = np.zeros(self.X.input_shape, np.bool)
        self.ever_active = self.updated_ever_active(unpenalized)

        if np.any(self.elastic_net_param[unpenalized]):
            warn('convention is that parameters with no l1 penalty have no Lagrange parameter in front '
                 'of their ridge term so that lambda_max is easily computed')

        if np.any(unpenalized):
            (self.final_step, 
             null_grad, 
             null_soln,
             null_linpred,
             _ ) = self.solve_subproblem(self.solution,
                                         np.nonzero(unpenalized)[0],
                                         self.BIG,
                                         tol=1.e-8)
            linear_predictor = null_linpred
            self.solution[unpenalized] = null_soln
        else:
            linear_predictor = np.zeros(self.saturated_loss.shape)

        self.grad_solution = (self.full_gradient(self.saturated_loss, 
                                                 linear_predictor) + self.enet_grad(self.solution, 
                                                                                    self._penalized_vars,
                                                                                    1))

    # LASSO specific part

    def enet_loss(self, 
                  lagrange,
                  candidate_set=None):

        return _restricted_elastic_net(self.elastic_net_param, 
                                       self.penalty.weights,
                                       lagrange,
                                       self.alpha,
                                       candidate_set)

    def subsample(self,
                  case_idx):
        '''

        Create a new path, by subsampling
        cases of `self.saturated_loss`.

        Case weights are computed
        with `self.saturated_loss.subsample`.

        Parameters
        ----------

        case_idx : index
            An index-like object used 
            to specify which cases to include
            in the subsample.

        Returns
        -------

        subsample_path : path object
            A path object with a modified smooth part
            reflecting the subsampling.

        '''
        subsample_loss = self.saturated_loss.subsample(case_idx)
        return self.__class__(subsample_loss,
                              self.X,
                              self.penalty.weights,
                              elastic_net_param=self.elastic_net_param,
                              alpha=self.alpha)

    def check_KKT(self,
                  grad_solution,
                  solution,
                  lagrange,
                  penalty=None):

        '''

        Check KKT conditions over
        the groups in the path.
        Returns boolean indicating
        which groups are failing the KKT conditions
        (these could be `active` groups or
        `inactive` groups).

        Parameters
        ----------

        grad_solution : ndarray
             Candidate for gradient of smooth loss at 
             Lagrange value `lagrange`.

        solution : ndarray
             Candidate for solution to problem 
             Lagrange value `lagrange`.

        lagrange : float
             Lagrange value for penalty

        penalty : object (optional)
             A weighted L1 penalty. If None, defaults
             to `self.penalty`.

        '''

        if penalty is None:
            penalty = self.penalty

        value = _check_KKT(penalty.weights, 
                           grad_solution, 
                           solution, 
                           lagrange)
        return value[0] > 0, value[1]

    def strong_set(self,
                   lagrange_cur,
                   lagrange_new,
                   grad_solution):

        bool = _strong_set(self.penalty.weights,
                          lagrange_cur,
                          lagrange_new,
                          grad_solution)
        cols = np.nonzero(bool)[0]
        return cols, bool, cols

    def active_set(self, solution):
        return solution != 0

    def restricted_penalty(self, subset):
        if subset is not None:
            weights = self.penalty.weights[subset]
        else:
            weights = self.penalty.weights
        return weighted_l1norm(weights,
                               lagrange=1)

    def solve_subproblem(self, 
                         solution, 
                         candidate_set, 
                         lagrange_new, 
                         **solve_args):
    
        # solve a problem with a candidate set

        sub_loss, sub_penalty, sub_X = _restricted_problem(self.X, 
                                                           self.saturated_loss, 
                                                           self.alpha, 
                                                           lagrange_new * self.penalty.weights, 
                                                           candidate_set,
                                                           self.subsample_columns)
        if self.alpha < 1:
            sub_elastic_net = self.enet_loss(lagrange_new,
                                             candidate_set)

            sub_loss = smooth_sum([sub_loss, sub_elastic_net])

        sub_problem = simple_problem(sub_loss, sub_penalty)
        sub_problem.coefs[:] = solution[candidate_set] # warm start
        sub_soln = sub_problem.solve(**solve_args)
        sub_grad = sub_loss.smooth_objective(sub_soln, mode='grad') 
        sub_linear_pred = sub_X.dot(sub_soln)
        candidate_bool = np.zeros(self.group_shape, np.bool)
        candidate_bool[candidate_set] = True
        return sub_problem.final_step, sub_grad, sub_soln, sub_linear_pred, candidate_bool

    def enet_grad(self,
                  solution,
                  penalized, # boolean
                  lagrange_new,
                  subset=None):

        weights = self.penalty.weights
        elastic_net_param = self.elastic_net_param
        if subset is not None:
            solution = solution[subset]
            penalized = penalized[subset]
            elastic_net_param = elastic_net_param[subset]
        G = (1 - self.alpha) * solution * elastic_net_param
        G[penalized] *= lagrange_new
        return G

    def updated_ever_active(self,
                            index_obj,
                            group_ids=False): # ignored as result is the same
        if not hasattr(self, '_ever_active'):
            self._ever_active = np.zeros(self.group_shape, np.bool)
        _ever_active = self._ever_active.copy()
        _ever_active[index_obj] = True
        return list(np.nonzero(_ever_active)[0])

    @property
    def unpenalized(self):
        """
        Unpenalized groups and variables.

        Returns
        -------

        groups : sequence
            Groups with weights equal to 0.

        variables : ndarray
            Boolean indicator that is True if no penalty on that variable.

        """
        unpen = self.penalty.weights == 0
        return np.nonzero(unpen)[0], unpen

# Some common loss factories

def logistic(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return lasso_path(glm.logistic_loglike(Y.shape, Y), X, *args, **keyword_args)

def probit(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return lasso_path(glm.probit_loglike(Y.shape, Y), X, *args, **keyword_args)

def cloglog(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return lasso_path(glm.cloglog_loglike(Y.shape, Y), X, *args, **keyword_args)

def gaussian(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return lasso_path(glm.gaussian_loglike(Y.shape, Y), X, *args, **keyword_args)

def cox(X, T, S, *args, **keyword_args):
    T, S = np.asarray(T), np.asarray(S)
    return lasso_path(glm.cox_loglike(T.shape, T, S), X, *args, **keyword_args)

def poisson(X, Y, *args, **keyword_args):
    Y = np.asarray(Y)
    return lasso_path(glm.poisson_loglike(Y.shape, Y), X, *args, **keyword_args)

def huber(X, Y, smoothing_parameter, *args, **keyword_args):
    Y = np.asarray(Y)
    return lasso_path(glm.huber_loss(Y.shape, Y, smoothing_parameter), X, *args, **keyword_args)

def huber_svm(X, Y, smoothing_parameter, *args, **keyword_args):
    Y = np.asarray(Y)
    return lasso_path(glm.huber_svm(Y.shape, Y, smoothing_parameter), X, *args, **keyword_args)

# private functions

def _get_lagrange_max(penalty,
                     grad_solution):
    penalized = penalty.weights > 0
    return np.fabs(grad_solution[penalized] / penalty.weights[penalized]).max()


def _strong_set(lasso_weights,
                lagrange_cur,
                lagrange_new,
                gradient,
                slope_estimate=1):
    """
    Return a Boolean indicator for each group
    indicating whether in the strong set or not.
    """
    thresh = 2 * lagrange_new - lagrange_cur
    scaled_grad = np.fabs(gradient)
    scaled_grad[lasso_weights > 0] /= lasso_weights[lasso_weights > 0]
    scaled_grad[lasso_weights == 0] = np.inf
    return scaled_grad > thresh

def _restricted_elastic_net(elastic_net_params, 
                            lasso_weights, 
                            lagrange, 
                            alpha, 
                            candidate_set=None):
    penalized = lasso_weights != 0
    new_params = elastic_net_params * (1 - alpha)
    new_params[penalized] *= lagrange 
    if candidate_set is not None:
        new_params = new_params[candidate_set]
    return quadratic_loss(new_params.shape,
                          new_params,
                          Qdiag=True)

def _restricted_problem(X, 
                        saturated_loss, 
                        alpha,
                        lasso_weights, 
                        candidate_set, 
                        subsample_columns):

    X_candidate = subsample_columns(X, candidate_set)
    restricted_loss = affine_smooth(saturated_loss, X_candidate)
    restricted_penalty = weighted_l1norm(lasso_weights[candidate_set] * alpha, lagrange=1.)

    return restricted_loss, restricted_penalty, X_candidate

def _check_KKT(lasso_weights, 
               grad, 
               solution, 
               lagrange, 
               tol=1.e-2):

    """
    Check whether (grad, solution) satisfy
    KKT conditions at a given tolerance.
    """

    ACTIVE = 1
    INACTIVE = 2
    UNPENALIZED = 3

    active = (solution != 0) * lasso_weights > 0
    active_failing = np.zeros(solution.shape, np.int)
    active_failing[active] = (np.fabs(grad[active] / (lagrange * lasso_weights[active]) + np.sign(solution[active])) > tol) * ACTIVE

    # inactive_failing = np.zeros(solution.shape, np.int) - 1
    # 
    # inactive_failing[inactive] = (np.fabs(grad[inactive] / (lagrange * lasso_weights[inactive])) > 1 + tol) * INACTIVE

    inactive = (solution == 0) * lasso_weights > 0
    inactive_terms = np.zeros(solution.shape)
    inactive_terms[inactive] = np.fabs(grad[inactive] / (lagrange * lasso_weights[inactive]))
    inactive_ranks = solution.shape[0] - rankdata(inactive_terms)
    inactive_ranks[inactive_terms <= 1+tol] = -1

    unpenalized = lasso_weights == 0
    active_failing[unpenalized] = (np.fabs(grad[unpenalized] / np.mean(lasso_weights[lasso_weights>0])) > tol) * UNPENALIZED

    np.testing.assert_allclose(inactive_ranks >= 0, inactive_terms > 1 + tol)

    return active_failing, inactive_ranks



import numpy as np
from scipy.stats import norm as normal_dbn

have_sklearn = True
try:
    from .base import (sklearn_regression,
                       sklearn_regression_lagrange,
                       sklearn_classifier,
                       sklearn_classifier_lagrange)
except:
    have_sklearn = False
from ..smooth.glm import (glm, 
                          probit_loglike)

if have_sklearn:
    class sklearn_probit(sklearn_regression):

        """

        A simple regression mixin for sklearn
        that allows any atom to be used as a regularizer.

        """

        def _loglike_factory(self, X, y):
            response, case_weights_, offset_ = self._check_y_arg(y)

            if response.ndim == 2:
                successes = response[:,0]
                trials = response[:,1]
            else:
                successes = response
                trials = None

            return glm.probit(X, 
                              successes,
                              trials=trials,
                              case_weights=case_weights_,
                              coef=self.coef,
                              saturated_offset=offset_)

        def _saturated_score(self,
                             predictions,
                             response,
                             case_weights=None):

            if response.ndim == 2:
                successes = response[:,0]
                trials = response[:,1]
            else:
                successes = response
                trials = None

            loss = lambda yhat: probit_loglike(successes.shape,
                                               successes,
                                               trials=trials,
                                               case_weights=case_weights).smooth_objective(yhat, 'func')

            # factor of 2 to form proper deviance (default is negative log-likelihood,
            # while deviance is 2 * negative log-likelihood
            # negative sign is to align with sklearn's maximizing a score with grid search

            if self.score_method == 'deviance':
                return -2 * loss(predictions)
            elif self.score_method == 'mean_deviance':
                return -2 * loss(predictions) / predictions.shape[0]
            elif self.score_method == 'R2':
                SSE = 2 * loss(predictions)
                pi_0 = response.mean()
                probit_0 = normal_dbn.ppf(pi_0)
                SST = 2 * loss(probit_0 * np.ones_like(response)) # X: correct for probit?
                return 1 - SSE / SST
            elif self.score_method == 'accuracy':
                labels = predictions > 0
                return np.mean(labels == response)
            else:
                return np.nan

    class sklearn_probit_lagrange(sklearn_regression_lagrange, sklearn_probit):
        pass

    class sklearn_probit_classifier(sklearn_classifier):

        """

        A simple regression mixin for sklearn
        that allows any atom to be used as a regularizer.

        """

        def _loglike_factory(self, X, y):
            response, case_weights_, offset_ = self._check_y_arg(y)

            if response.ndim == 2:
                successes = response[:,0]
                trials = response[:,1]
            else:
                successes = response
                trials = None

            return glm.probit(X, 
                              successes,
                              trials=trials,
                              case_weights=case_weights_,
                              coef=self.coef,
                              saturated_offset=offset_)

        def _saturated_score(self,
                             predictions,
                             response,
                             case_weights=None):

            if response.ndim == 2:
                successes = response[:,0]
                trials = response[:,1]
            else:
                successes = response
                trials = None

            if self.score_method == 'accuracy':
                return np.mean(predictions == successes)
            return np.nan

        def predict_proba(self, X):
            """
            Predict new probabilities in classification setting.

            Parameters
            ----------

            X : np.ndarray((n, p))
                Feature matrix.

            Returns
            -------

            probs : np.ndarray(n)
                Predictions from classification model.

            """
            linpred = X.dot(self._coefs)
            return normal_dbn.cdf(linpred)

    class sklearn_probit_classifier_lagrange(sklearn_probit_classifier):
        pass




import numpy as np
from scipy import sparse

from ..algorithms import FISTA
from ..problems.composite import (composite, nonsmooth as nonsmooth_composite,
                        smooth as smooth_composite)
from ..affine import (vstack as afvstack, identity as afidentity, power_L,
                     selector as afselector)
from ..problems.separable import separable
from ..problems.dual_problem import dual_problem, stacked_dual
from ..atoms import affine_atom as nonsmooth_affine_atom
from ..atoms.cones import zero_constraint, zero as zero_nonsmooth, affine_cone

from ..identity_quadratic import identity_quadratic

class container(composite):
    """
    A container class for storing/combining seminorm_atom classes

    Notes
    -----

    This function copies the nonsmooth atoms
    """

    def __init__(self, *atoms, **keywords):
        self.nonsmooth_atoms = []
        self.smooth_atoms = []
        for atom in atoms:
            if (isinstance(atom, nonsmooth_composite) or 
                isinstance(atom, nonsmooth_affine_atom) or
                isinstance(atom, affine_cone)):
                self.nonsmooth_atoms.append(atom)
            elif isinstance(atom, smooth_composite):
                self.smooth_atoms.append(atom)
            else:
                raise ValueError('each atom should either be a smooth or nonsmooth composite')

        if len(self.nonsmooth_atoms) == 0 and len(self.smooth_atoms) == 0:
            raise ValueError('must specify some atoms')

        if len(self.nonsmooth_atoms) == 0:
            self.nonsmooth_atoms = [zero_nonsmooth(self.smooth_atoms[0].shape)]

        self.transform, self.atom = stacked_dual(self.smooth_atoms[0].shape, *self.nonsmooth_atoms)
        self.coefs = np.zeros(self.transform.input_shape)

        self.smoothq = identity_quadratic(0,0,0,0)
        for atom in self.smooth_atoms:
            self.smoothq += atom.quadratic

    def smooth_objective(self, x, mode='both', check_feasibility=False):
        """
        The smooth_objective DOES NOT INCLUDE the identity
        quadratic of all the smooth atoms.
        """
        value, grad = 0, np.zeros(x.shape)
        if mode == 'func':
            for atom in self.smooth_atoms:
                value += atom.smooth_objective(x, mode=mode, 
                                               check_feasibility=check_feasibility)
            return value
        elif mode == 'both':
            for atom in self.smooth_atoms:
                v, g = atom.smooth_objective(x, mode=mode, 
                                             check_feasibility=check_feasibility)
                value += v
                grad += g
            return value, grad

        elif mode == 'grad':
            for atom in self.smooth_atoms:
                grad += atom.smooth_objective(x, mode=mode, 
                                              check_feasibility=check_feasibility)
            return grad
        else:
            raise ValueError("Mode specified incorrectly")

    def nonsmooth_objective(self, x, check_feasibility=False):
        out = 0.
        for atom in self.nonsmooth_atoms + self.smooth_atoms:
            out += atom.nonsmooth_objective(x,
                                            check_feasibility=check_feasibility)
        out += self.quadratic.objective(x, 'func')
        return out

    default_solver = FISTA
    def proximal(self, proxq, prox_control=None):
        """
        The proximal function for the primal problem
        """

        transform, atom = self.transform, self.atom

        if not (isinstance(transform, afidentity) or
                isinstance(transform, afselector)):
            #Default fitting parameters
            prox_defaults = {'max_its': 5000,
                             'min_its': 5,
                             'return_objective_hist': False,
                             'tol': 1e-14,
                             'debug':False}

            if prox_control is not None:
                prox_defaults.update(prox_control)
            prox_control = prox_defaults

            primal_objective = zero_nonsmooth(transform.input_shape)
            primal_objective.quadratic = proxq + self.quadratic + self.smoothq
            dual_objective = primal_objective.conjugate
            dualp = dual_problem(dual_objective,
                                 transform,
                                 atom)

            #Approximate Lipschitz constant
            if not 'dual_reference_lipschitz' in prox_control.keys():
                # shouldn't do this over and over
                self.dual_reference_lipschitz = 1.05*power_L(transform, debug=prox_control['debug'])
            else:
                self.dual_reference_lipschitz = prox_control['dual_reference_lipschitz']
                prox_control.pop('dual_reference_lipschitz')
                
            dualopt = container.default_solver(dualp)
            dualopt.perform_backtrack = False
            dualopt.debug = prox_control['debug']

            if dualopt.perform_backtrack:
                #If backtracking set start_step
                prox_control['start_step'] = proxq.coef / self.dual_reference_lipschitz 

            # the lipschitz estimate comes from the
            # fact that the conjugate of a quadratic
            # with coef lipschitz is quadratic with coef 1/lipschitz

            dualp.lipschitz = self.dual_reference_lipschitz / proxq.coef

            if hasattr(self, 'dual_minimizer'):
                dualopt.composite.coefs[:] = self.dual_minimizer
            history = dualopt.fit(**prox_control)
            dual_minimizer = dualopt.composite.coefs
            lipschitz, x, grad = proxq.coef, proxq.center, proxq.linear_term
            if prox_control['return_objective_hist']:
                return x - grad / lipschitz - transform.adjoint_map(dualopt.composite.coefs/lipschitz), history
            else:
                return x - grad / lipschitz - transform.adjoint_map(dualopt.composite.coefs/lipschitz)

        else:
            proxq = proxq + self.quadratic + self.smoothq
            primal = atom.conjugate
            lipschitz, x, grad = proxq.coef, proxq.center, proxq.linear_term
            if isinstance(transform, afselector):
                z = x - grad / lipschitz
                z[transform.index_obj] = primal.proximal(proxq[transform.index_obj])
                return z
            else:
                return primal.proximal(proxq)

    def solve(self, quadratic=None, return_optimum=False, **fit_args):
        if quadratic is not None:
            oldq, newq = self.quadratic, self.quadratic + quadratic
        else:
            oldq = newq = self.quadratic

        solver = FISTA(self)
        solver.fit(**fit_args)

        if return_optimum:
            value = (self.objective(self.coefs), self.coefs)
        else:
            value = self.coefs
        self.quadratic = oldq
        return value




from __future__ import print_function, division, absolute_import

from warnings import warn
import gc

import numpy as np
import numpy.linalg as npl

from .lasso import lasso_path

import scipy.sparse

from ..affine import power_L, normalize, selector, identity, adjoint
from ..atoms.seminorms import l1norm, constrained_positive_part
from ..smooth import sum as smooth_sum, affine_smooth
from ..smooth.glm import glm
from ..problems.simple import simple_problem
from ..identity_quadratic import identity_quadratic as iq
from ..atoms.mixed_lasso import mixed_lasso, strong_set as strong_set_ml, check_KKT

# Constants used below

UNPENALIZED = -1
L1_PENALTY = -2
POSITIVE_PART = -3
NONNEGATIVE = -4

class mixed_lasso_path(lasso_path):

    def __init__(self, 
                 loss_factory, 
                 X, 
                 penalty_structure=None, 
                 group_weights={},
                 elastic_net=iq(0,0,0,0),
                 alpha=0., intercept=True,
                 positive_part=None,
                 unpenalized=None,
                 lagrange_proportion = 0.05,
                 nstep = 100,
                 scale=True,
                 center=True):


        self.loss_factory = loss_factory

        self.scale = scale
        self.center = center

        # for group lasso weights, if implied by penalty_structure
        self.group_weights = group_weights

        # normalize X, adding intercept if needed
        self.intercept = intercept
        p = X.shape[1]
        if self.intercept:
            self.penalty_structure = np.ones(p+1) * L1_PENALTY
            self.penalty_structure[0] = UNPENALIZED
            if penalty_structure is not None:
                self.penalty_structure[1:] = penalty_structure

            if scipy.sparse.issparse(X):
                self._X1 = scipy.sparse.hstack([np.ones((X.shape[0], 1)), X]).tocsc() 
            else:
                self._X1 = np.hstack([np.ones((X.shape[0], 1)), X])
            if self.scale or self.center:
                self._Xn = normalize(self._X1, center=self.center, scale=self.scale, intercept_column=0)
                which_0 = self._Xn.col_stds == 0
            else:
                self._Xn = self._X1
                which_0 = np.zeros(self._Xn.shape)

        else:
            self.penalty_structure = np.ones(p) * L1_PENALTY
            if penalty_structure is not None:
                self.penalty_structure[:] = penalty_structure

            if self.scale or self.center:
                self._Xn = normalize(X, center=self.center, scale=self.scale)
                which_0 = self._Xn.col_stds == 0
            else:
                self._Xn = X
                which_0 = np.zeros(self._Xn.shape)

        if np.any(which_0):
            self._selector = selector(~which_0, self._Xn.input_shape)
            if self.scale or self.center:
                self._Xn = self._Xn.slice_columns(~which_0)
            else:
                self._Xn = self._Xn[:,~which_0]
        else:
            if self.scale or self.center:
                self._selector = identity(self._Xn.input_shape)
            else:
                self._selector = identity(self._Xn.shape)

        # the penalty parameters
        self.alpha = alpha
        self.lagrange_proportion = lagrange_proportion
        self.nstep = nstep
        self._elastic_net = elastic_net.collapsed()

        self.initial_active = (np.equal(self.penalty_structure, UNPENALIZED) + 
                               np.equal(self.penalty_structure, NONNEGATIVE))
        self.ever_active = self.initial_active.copy()

    @property
    def shape(self):
        if self.scale or self.center:
            return self.Xn.output_shape[0], self.Xn.input_shape[0]
        else:
            return self.Xn.shape

    @property
    def nonzero(self):
        return self._selector

    @property
    def elastic_net(self):
        q = self._elastic_net
        q.coef *= self.lagrange
        q.linear_term *= self.lagrange
        return q

    @property
    def Xn(self):
        return self._Xn

    @property
    def loss(self):
        if not hasattr(self, '_loss'):
            self._loss = self.loss_factory(self._Xn)
        return self._loss

    @property
    def null_solution(self):
        if not hasattr(self, "_null_soln"):
            n, p = self.shape
            self._null_soln = np.zeros(p)
            null_problem, null_selector = self.restricted_problem(self.initial_active, self.lagrange_max)[:2]
            self._null_soln = null_selector.adjoint_map(null_problem.solve())
        return self._null_soln

    @property
    def lagrange_max(self):
        if not hasattr(self, "_lagrange_max"):
            null_soln = self.null_solution
            null_grad = self.loss.smooth_objective(null_soln, 'grad')
            self.penalty = mixed_lasso(self.penalty_structure, 1., weights=self.group_weights)
            conj = self.penalty.conjugate
            self._lagrange_max = conj.seminorm(null_grad)

        return self._lagrange_max

    def get_lagrange_sequence(self):
        if not hasattr(self, "_lagrange_sequence"):
            self._lagrange_sequence = self.lagrange_max * np.exp(np.linspace(np.log(self.lagrange_proportion), 0, 
                                                                             self.nstep))[::-1]
        return self._lagrange_sequence

    def set_lagrange_sequence(self, lagrange_sequence):
        self._lagrange_sequence = lagrange_sequence
    
    lagrange_sequence = property(get_lagrange_sequence, set_lagrange_sequence)

    @property
    def problem(self):
        p = self.shape[1]
        if not hasattr(self, "_problem"):
            self._problem = self.restricted_problem(np.ones(self.shape[1], np.bool),
                                                    self.lagrange_max)[0]
        return self._problem

    def get_lagrange(self):
        return self._problem.proximal_atom.lagrange

    def set_lagrange(self, lagrange):
        proximal_atom = self._problem.proximal_atom
        proximal_atom.lagrange = lagrange
        self.penalty.lagrange = lagrange
    lagrange = property(get_lagrange, set_lagrange)

    @property
    def solution(self):
        return self.problem.coefs

    @property
    def active(self):
        return self.solution != 0

    @property
    def lipschitz(self):
        if not hasattr(self, "_lipschitz"):
            self._lipschitz = power_L(self.Xn)
        return self._lipschitz

    def grad(self, loss=None):
        '''
        Gradient at current value. This includes the gradient
        of the smooth loss as well as the gradient of the elastic net part.
        This is used for determining whether the KKT conditions are met
        and which coefficients are in the strong set.
        '''
        if loss is None:
            loss = self.loss
        gsmooth = self.loss.smooth_objective(self.solution, 'grad')
        penalized = self.penalty_structure != UNPENALIZED
        # XXX the elastic net is probably not quite right here if the elastic net has a non-zero center
        gquad = self.elastic_net.objective(self.solution[penalized], 'grad')
        gsmooth[penalized] += gquad

        return gsmooth

    def strong_set(self, lagrange_cur, lagrange_new, grad=None,
                   slope_estimate=1):
        if grad is None:
            grad = self.grad()

        return strong_set_ml(self.penalty, lagrange_cur, lagrange_new, grad, slope_estimate)

    def slice_columns(self, columns):
        if self.scale or self.center:
            Xslice = self.Xn.slice_columns(columns)
        else:
            Xslice = self.Xn[:,columns]
        return Xslice

    def construct_loss(self, candidate_set, lagrange):
        Xslice = self.slice_columns(candidate_set)
        loss = self.loss_factory(Xslice)
        if self.intercept:
            Xslice.intercept_column = 0
        return Xslice, loss

    def restricted_problem(self, candidate_set, lagrange):
        '''
        Assumes the candidate set includes intercept as first column.
        '''

        Xslice, loss = self.construct_loss(candidate_set, lagrange)

        restricted_penalty_structure = self.penalty_structure[candidate_set]
        rps = restricted_penalty_structure # shorthand

        sliced_penalty = mixed_lasso(rps, lagrange, weights=self.group_weights)
        problem_sliced = simple_problem(loss, sliced_penalty)
        candidate_selector = selector(candidate_set, self.shape[1])
        return problem_sliced, candidate_selector, restricted_penalty_structure

    def solve_subproblem(self, candidate_set, lagrange_new, **solve_args):
    
        # try to solve the problem with the active set
        subproblem, selector, penalty_structure = self.restricted_problem(candidate_set, lagrange_new)
        subproblem.coefs[:] = selector.linear_map(self.solution)
        sub_soln = subproblem.solve(**solve_args)
        self.solution[:] = selector.adjoint_map(sub_soln)

        grad = subproblem.smooth_objective(sub_soln, mode='grad') 
        self.final_step = subproblem.final_step
        return self.final_step, grad, sub_soln, penalty_structure

    def main(self, inner_tol=1.e-5, verbose=False):

        # scaling will be needed to get coefficients on original scale   
        if self.scale:
            scalings = np.asarray(self.Xn.col_stds).reshape(-1)
        else:
            scalings = np.ones(self.shape[1])
        scalings = self.nonzero.adjoint_map(scalings)

        # take a guess at the inverse step size
        self.final_step = 1000. / self.lipschitz 
        lseq = self.lagrange_sequence # shorthand

        # first solution corresponding to all zeros except intercept 

        self.solution[:] = self.null_solution.copy()

        grad_solution = self.grad().copy()
        strong, strong_selector = self.strong_set(lseq[0], lseq[1], grad=grad_solution)

        p = self.shape[0]

        rescaled_solutions = scipy.sparse.csr_matrix(self.nonzero.adjoint_map(self.solution) 
                                                     / scalings)

        objective = [self.loss.smooth_objective(self.solution, 'func')]
        # not quite right -- should check tight constraints
        dfs = [np.sum(self.initial_active)]
        retry_counter = 0

        all_failing = np.zeros(grad_solution.shape, np.bool)

        for lagrange_new, lagrange_cur in zip(lseq[1:], lseq[:-1]):
            self.lagrange = lagrange_new
            tol = inner_tol
            active_old = self.active.copy()
            num_tries = 0
            debug = False
            coef_stop = True
            while True:
                strong, strong_selector = self.strong_set(lagrange_cur, 
                                                          lagrange_new, grad=grad_solution)

                subproblem_set = self.ever_active + all_failing
                final_step, grad, sub_soln, penalty_structure \
                    = self.solve_subproblem(subproblem_set,
                                            lagrange_new,
                                            tol=tol,
                                            start_step=self.final_step,
                                            debug=debug and verbose,
                                            coef_stop=coef_stop)

                p = self.shape[1]

                self.solution[subproblem_set][:] = sub_soln
                # this only corrects the gradient on the subproblem_set
                grad_solution[subproblem_set][:] = grad

                strong_problem = self.restricted_problem(strong, lagrange_new)[0]
                strong_soln = self.solution[strong]
                strong_grad = (strong_problem.smooth_objective(strong_soln, mode='grad') + 
                               self.elastic_net[strong].objective(strong_soln, mode='grad'))
                strong_penalty = strong_problem.proximal_atom

                strong_failing = check_KKT(strong_penalty, strong_grad, strong_soln, lagrange_new) 

                if np.any(strong_failing):
                    all_failing += (
                        strong_selector.adjoint_map(strong_failing) != 0)
                else:
                    self.solution[subproblem_set][:] = sub_soln
                    grad_solution = self.grad()
                    all_failing = check_KKT(self.penalty, grad_solution, self.solution, lagrange_new)

                    if not all_failing.sum():
                        self.ever_active += self.solution != 0
                        self.final_step = final_step
                        break
                    else:
                        if verbose:
                            print('failing:', np.nonzero(all_failing)[0])
                        retry_counter += 1
                        self.ever_active += all_failing

                tol /= 2.
                num_tries += 1
                if num_tries % 5 == 0:

                    self.solution[subproblem_set][:] = sub_soln
                    self.solution[~subproblem_set][:] = 0
                    grad_solution = self.grad()

                    debug = True
                    tol = inner_tol
                    if num_tries >= 10:
                        warn('convergence not achieved for lagrange=%0.4e' % lagrange_new)
                        break

            rescaled_solution = self.nonzero.adjoint_map(self.solution)
            rescaled_solutions = scipy.sparse.vstack([rescaled_solutions, rescaled_solution])
            objective.append(self.loss.smooth_objective(self.solution, mode='func'))
            dfs.append(self.ever_active.shape[0])
            gc.collect()

            if verbose:
                print(lagrange_cur / self.lagrange_max,
                      lagrange_new,
                      (self.solution != 0).sum(),
                      1. - objective[-1] / objective[0],
                      list(self.lagrange_sequence).index(lagrange_new),
                      np.fabs(rescaled_solution).sum())

        objective = np.array(objective)
        output = {'devratio': 1 - objective / objective.max(),
                  'df': dfs,
                  'lagrange': self.lagrange_sequence,
                  'scalings': scalings,
                  'beta':rescaled_solutions.T}

        return output

    # Some common loss factories

    @classmethod
    def logistic(cls, X, Y, *args, **keyword_args):
        return cls(lambda Xs: glm.logistic(Xs, Y), X, *args, **keyword_args)

    @classmethod
    def gaussian(cls, X, Y, *args, **keyword_args):
        return cls(lambda Xs: glm.gaussian(Xs, Y), X, *args, **keyword_args)




import numpy as np

from ...atoms.seminorms import l1norm
from ...smooth.glm import glm
from ..newton import quasi_newton
from ..simple import simple_problem

def test_lagrange():

    n, p, s = 1000, 50, 5
    X = np.random.standard_normal((n, p))
    beta = np.zeros(p)
    beta[:s] = 20 * np.random.standard_normal(s) / np.sqrt(n)
    eta = X.dot(beta)
    pi = np.exp(eta) / (1 + np.exp(eta))
    Y = np.random.binomial(1, pi)
    assert(Y.shape == pi.shape)
    loss = glm.logistic(X, Y)
    penalty = l1norm(p, lagrange=4)

    qn = quasi_newton(loss,
                      penalty,
                      X.T.dot(X) / 4.)
    soln_newton = qn.solve(niter=1000, tol=1.e-6,
                           maxfun=5, maxiter=5)

    problem = simple_problem(loss, penalty)
    soln_simple = problem.solve(min_its=200, tol=1.e-14)

    assert(np.linalg.norm(soln_newton - soln_simple) / np.linalg.norm(soln_simple) < 1.e-6)

def test_bound():

    n, p = 1000, 50
    X = np.random.standard_normal((n, p))
    Y = np.random.binomial(1, 0.5, size=(n,))
    loss = glm.logistic(X, Y)
    penalty = l1norm(p, bound=0.5)

    qn = quasi_newton(loss,
                      penalty,
                      X.T.dot(X) / 4.)
    soln_newton = qn.solve(niter=1000, tol=1.e-10,
                           maxfun=5, maxiter=5)

    problem = simple_problem(loss, penalty)
    soln_simple = problem.solve(tol=1.e-14)

    assert(np.linalg.norm(soln_newton - soln_simple) / max(np.linalg.norm(soln_simple), 1) < 1.e-5)
    assert(np.fabs(problem.objective(soln_newton) - problem.objective(soln_simple)) < 1.e-6)



r"""
This module contains a single class that is meant to represent
a quadratic of the form

.. math::

   \frac{L}{2} \|x-\mu\|^2_2 + \langle \eta, x \rangle + \gamma

with :math:`L, \mu, \eta, \gamma` = (coef, center, linear_term,
constant_term).
"""

from copy import copy

from numpy.linalg import norm
from numpy import all, asarray, allclose

class identity_quadratic(object):

    r"""
    This object is a quadratic function

    .. math::

        x \mapsto \frac{L}{2} \|x-\mu\|^2_2 + \langle \eta, x \rangle + \gamma

    used in the proximal methods of all atoms.

    """

    objective_vars = {'var': r'\beta', 'shape':'p', 'offset':r'\alpha'}

    def __eq__(self, other):
        if isinstance(other, identity_quadratic):
            return (allclose(self.coef, other.coef) and
                    allclose(self.center, other.center) and
                    allclose(self.linear_term, other.linear_term) and
                    allclose(self.constant_term, other.constant_term))

    def __init__(self, coef, center, linear_term, constant_term=0):

        if center is not None:
            center = asarray(center)
            if center.shape == ():
                center = float(center)
        if linear_term is not None:
            linear_term = asarray(linear_term)
            if linear_term.shape == ():
                linear_term = float(linear_term)

        if coef is None:
            self.coef = 0.
        else:
            self.coef = coef

        self.center = center
        self.linear_term = linear_term
        if constant_term is None:
            self.constant_term = 0
        else:
            self.constant_term = constant_term

    @property
    def iszero(self):
        return all([self.coef in [0, None],
                    self.center is None or all(self.center == 0),
                    self.linear_term is None or all(self.linear_term == 0),
                    self.constant_term in [0, None]])

    def __copy__(self):
        return identity_quadratic(self.coef,
                                  copy(self.center),
                                  copy(self.linear_term),
                                  copy(self.constant_term))

    def noneify(self):
        '''
        replace zeros with nones
        '''
        if self.coef is None:
            self.coef = 0
        if self.constant_term is None:
            self.constant_term = 0
        if self.linear_term is not None and all(self.linear_term == 0):
            self.linear_term = None
        if self.center is not None and all(self.center_term == 0):
            self.center_term = None

    def zeroify(self):
        for a in ['coef', 'center', 'linear_term', 'constant_term']:
            if getattr(self, a) is None:
                setattr(self, a, 0)

    def recenter(self, offset):

        if offset is not None and all(offset == 0):
            offset = None

        if offset is not None:
            cpq = copy(self)
            cpq.center -= offset 
            cpq = cpq.collapsed()
            return offset, cpq
        else:
            return None, self.collapsed()


    def objective(self, x, mode='both'):
        coef, center, linear_term = self.coef, self.center, self.linear_term
        cons = self.constant_term
        if linear_term is None:
            linear_term = 0
        if center is not None:
            r = x - center
        else:
            r = x
        if mode == 'both':
            if linear_term is not None:
                return (norm(r)**2 * coef / 2. + (linear_term * x).sum() 
                        + cons, coef * r + linear_term)
            else:
                return (norm(r)**2 * coef / 2. + cons,
                        coef * r)
        elif mode == 'func':
            if linear_term is not None:
                return norm(r)**2 * coef / 2. + (linear_term * x).sum() + cons
            else:
                return norm(r)**2 * coef / 2. + cons
        elif mode == 'grad':
            if linear_term is not None:
                return coef * r + linear_term
            else:
                return coef * r
        else:
            raise ValueError("Mode incorrectly specified")

    def __repr__(self):
        return 'identity_quadratic(%f, %s, %s, %f)' % (self.coef, repr(self.center), repr(self.linear_term), self.constant_term)

    def __add__(self, other):
        """ Return an identity quadratic given by the sum in the obvious way.

        It has center of 0, would be nice to have None, but there are some
        places we are still multiplying by -1
        """
        if not (other is None or isinstance(other, identity_quadratic)):
            raise ValueError('can only add None or other identity_quadratic')


        if other is None:
            return self
        else:
            sc = self.collapsed()
            oc = other.collapsed()
            newq = identity_quadratic(sc.coef + oc.coef, 0, 
                                      sc.linear_term + oc.linear_term,
                                      sc.constant_term + oc.constant_term)
            return newq 

    def get_shapes(self):
        '''
        Determine shape of any pieces and make sure they agree
        '''
        self.zeroify()
        lt = asarray(self.linear_term)
        center = asarray(self.center)

        if lt.shape != () and center.shape != ():
            if lt.shape != center.shape:
                raise ValueError('conflicting shapes of linear_term and center')
        return lt.shape, center.shape


    def __getitem__(self, slice):
        '''
        Return a new quadratic restricted to the variables in slice
        with constant_term=0.
        '''
        lts, cts = self.get_shapes()

        if lts != ():
            lt = self.linear_term[slice]
        else:
            lt = self.linear_term

        if cts != ():
            ct = self.center[slice]
        else:
            ct = self.center
        return identity_quadratic(self.coef,
                                  ct, lt, 0)

    def collapsed(self):
        """
        Return an identity quadratic with center of 0,
        would be nice to have None, but there are some 
        places we are still multiplying by -1
        """
        coef = 0 if self.coef is None else self.coef
        linear_term = 0.
        constant_term = self.constant_term
        if constant_term is None:
            constant_term = 0.
        if self.center is not None:
            linear_term -= coef * self.center
            constant_term += coef * norm(self.center)**2 / 2.
        if self.linear_term is not None:
            linear_term += self.linear_term

        return identity_quadratic(coef, 0, linear_term, constant_term)

    def latexify(self, var=r'\beta', idx=''):

        template_dict = copy(self.objective_vars)
        template_dict['idx'] = idx
        if var is not None:
            template_dict['var'] = var
        var = template_dict['var']
        idx = template_dict['idx']

        self.zeroify()
        terms = []
        if self.coef != 0:
            if not all(self.center == 0):
                terms.append(r'\frac{L_{%s}}{2}' % idx + r'\|%s-\mu_{%s}\|^2_2' % (var, idx))
            else:
                terms.append(r'\frac{L_{%s}}{2}' % idx + r'\|%s\|^2_2' % var)
        if self.linear_term is not None and not all(self.linear_term == 0):
            terms.append(r'\left \langle \eta_{%s}, %s \right \rangle' % (idx, var))
        if self.constant_term is not None and self.constant_term != 0:
            terms.append(r'\gamma_{%s} ' % idx)
        return ' + '.join(terms)

    def _repr_latex_(self):
        return r'''\begin{equation*} %s \end{equation*} ''' % self.latexify()

    @property
    def true_center(self):
        q = self.collapsed()
        if q.coef > 0:
            return -q.linear_term / q.coef
        return None

    @property
    def conjugate(self):
        a = self.collapsed()
        if a.coef != 0:
            return identity_quadratic(1./a.coef,
                                      a.linear_term,
                                      0,
                                      -a.constant_term)
        else:
            # possible import problem here
            from ..atoms.cones import zero_constraint
            q = identity_quadratic(0,0,0,-a.constant_term)
            return zero_constraint(a.linear_term.shape, offset=-a.linear_term, quadratic=q)



import numpy as np

have_sklearn = True
try:
    from .base import (sklearn_regression,
                       sklearn_regression_lagrange)
except ImportError:
    have_sklearn = False
from ..smooth.glm import (glm, 
                          cox_loglike)

if have_sklearn:
    class sklearn_cox(sklearn_regression):

        """

        A simple regression mixin for sklearn
        that allows any atom to be used as a regularizer.

        """

        def _loglike_factory(self, X, y):
            response, case_weights_, offset_ = self._check_y_arg(y)

            event_times = response[:,0]
            censoring = response[:,1]
            return glm.cox(X, 
                           event_times,
                           censoring,
                           case_weights=case_weights_,
                           coef=self.coef,
                           saturated_offset=offset_)

        def _saturated_score(self,
                             predictions,
                             response,
                             case_weights=None):

            event_times = response[:,0]
            censoring = response[:,1]

            loss = lambda yhat: cox_loglike(event_times.shape,
                                            event_times,
                                            censoring,
                                            case_weights=case_weights).smooth_objective(yhat, 'func')

            # factor of 2 to form proper deviance (default is negative log-likelihood,
            # while deviance is 2 * negative log-likelihood
            # negative sign is to align with sklearn's maximizing a score with grid search

            if self.score_method == 'deviance':
                return -2 * loss(predictions)
            elif self.score_method == 'mean_deviance':
                return -2 * loss(predictions) / predictions.shape[0]
            elif self.score_method == 'R2':
                SSE = 2 * loss(predictions)
                SST = loss(response.mean() * np.ones(response.shape[0])) # X: correct for Cox?
                return 1 - SSE / SST
            elif self.score_method == 'C-index':
                return np.nan
            else:
                return np.nan

    class sklearn_cox_lagrange(sklearn_regression_lagrange, sklearn_cox):

        """

        A simple regression mixin for sklearn
        that allows any atom to be used as a regularizer.

        """
        pass




from copy import copy

import nose.tools as nt
import numpy as np

from .. import glm
from ...affine.block_maps import block_columns

def test_poisson():

    X = np.random.standard_normal((10,5))
    Y = np.random.poisson(10, size=(10,))

    for case_weights in [np.ones(10), None]:
        L = glm.glm.poisson(X, Y, case_weights=case_weights)
        L.smooth_objective(np.zeros(L.shape), 'both')
        L.hessian(np.zeros(L.shape))

        Lcp = copy(L)
        L_sub = L.subsample(np.arange(5))

        sat_sub = L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works
        sat_sub.smooth_objective(np.zeros(sat_sub.shape))

        np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                   X.T.dot(1 - Y))
        np.testing.assert_allclose(L.hessian(np.zeros(L.shape)),
                                   X.T.dot(X))

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.saturated_loss.data = Y
        L.saturated_loss.data

        L.data = (X, Y)
        L.data

        # check that subsample is getting correct answer

        Xsub = X[np.arange(5)]
        Ysub = Y[np.arange(5)]

        beta = np.ones(L.shape)
        if case_weights is not None:
            Lsub2 = glm.glm.poisson(Xsub, Ysub, case_weights=case_weights[np.arange(5)])
            Lsub3 = glm.glm.poisson(Xsub, Ysub, case_weights=case_weights[np.arange(5)])
        else:
            Lsub2 = glm.glm.poisson(Xsub, Ysub)
            Lsub3 = glm.glm.poisson(Xsub, Ysub)

        Lsub3.coef *= 2.

        f2, g2 = Lsub2.smooth_objective(beta, 'both')
        f3, g3 = Lsub3.smooth_objective(beta, 'both')

        np.testing.assert_allclose(f3, 2*f2)
        np.testing.assert_allclose(g3, 2*g2)

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Lsub2.gradient(beta))

        np.testing.assert_allclose(L.gradient(beta),
                                   X.T.dot(L.saturated_loss.mean_function(X.dot(beta)) - Y))

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Xsub.T.dot(L_sub.saturated_loss.mean_function(Xsub.dot(beta)) - Ysub))

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Xsub.T.dot(Lsub2.saturated_loss.mean_function(Xsub.dot(beta)) - Ysub))


def test_gaussian():

    X = np.random.standard_normal((10,5))
    Y = np.random.standard_normal(10)

    for case_weights in [np.ones(10), None]:
        L = glm.glm.gaussian(X, Y, case_weights=case_weights)
        L.hessian(np.zeros(L.shape))
        L.smooth_objective(np.zeros(L.shape), 'both')

        sat_sub = L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works
        sat_sub.smooth_objective(np.zeros(sat_sub.shape))

        L_sub = L.subsample(np.arange(5))

        Xs = X[np.arange(5)]
        Ys = Y[np.arange(5)]

        beta = np.ones(5)
        value_sub = 0.5 * np.linalg.norm(Ys - Xs.dot(beta))**2
        grad_sub = Xs.T.dot(Xs.dot(beta) - Ys)

        f, g= L_sub.smooth_objective(beta, 'both')

        np.testing.assert_allclose(value_sub, f)
        np.testing.assert_allclose(grad_sub, g)

        np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                   -X.T.dot(Y))
        np.testing.assert_allclose(L.hessian(np.zeros(L.shape)),
                                   X.T.dot(X))

        Lcp = copy(L)

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.saturated_loss.data = Y
        L.saturated_loss.data

        L.data = (X, Y)
        L.data

        # check that subsample is getting correct answer

        Xsub = X[np.arange(5)]
        Ysub = Y[np.arange(5)]

        beta = np.ones(L.shape)
        if case_weights is not None:
            Lsub2 = glm.glm.gaussian(Xsub, Ysub, case_weights=case_weights[np.arange(5)])
            Lsub3 = glm.glm.gaussian(Xsub, Ysub, case_weights=case_weights[np.arange(5)])
        else:
            Lsub2 = glm.glm.gaussian(Xsub, Ysub)
            Lsub3 = glm.glm.gaussian(Xsub, Ysub)

        Lsub3.coef *= 2.

        f2, g2 = Lsub2.smooth_objective(beta, 'both')
        f3, g3 = Lsub3.smooth_objective(beta, 'both')

        np.testing.assert_allclose(f3, 2*f2)
        np.testing.assert_allclose(g3, 2*g2)

        beta = np.ones(L.shape)

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Lsub2.gradient(beta))

        np.testing.assert_allclose(L.gradient(beta),
                                   X.T.dot(L.saturated_loss.mean_function(X.dot(beta)) - Y))

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Xsub.T.dot(L_sub.saturated_loss.mean_function(Xsub.dot(beta)) - Ysub))

        np.testing.assert_allclose(L_sub.gradient(beta),
                                   Xsub.T.dot(Lsub2.saturated_loss.mean_function(Xsub.dot(beta)) - Ysub))

def test_huber():

    X = np.random.standard_normal((10,5))
    Y = np.random.standard_normal(10)

    for case_weights in [np.ones(10), None]:
        L = glm.glm.huber(X, Y, 0.1, case_weights=case_weights)
        L.smooth_objective(np.zeros(L.shape), 'both')

        Lcp = copy(L)
        L_sub = L.subsample(np.arange(5))

        sat_sub = L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works
        sat_sub.smooth_objective(np.zeros(sat_sub.shape))

        L.gradient(np.zeros(L.shape))
        nt.assert_raises(NotImplementedError, L.hessian, np.zeros(L.shape))

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.saturated_loss.data = Y
        L.saturated_loss.data

        L.data = (X, Y)
        L.data

def test_huber_svm():

    X = np.random.standard_normal((10,5))
    Y = np.random.binomial(1,0.5,size=(10,))

    for case_weights in [np.ones(10), None]:
        L = glm.glm.huber_svm(X, Y, 0.1, case_weights=case_weights)
        L.smooth_objective(np.zeros(L.shape), 'both')

        Lcp = copy(L)
        L_sub = L.subsample(np.arange(5))

        sat_sub = L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works
        sat_sub.smooth_objective(np.zeros(sat_sub.shape))

        L.gradient(np.zeros(L.shape))
        nt.assert_raises(NotImplementedError, L.hessian, np.zeros(L.shape))

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.saturated_loss.data = Y
        L.saturated_loss.data

        L.data = (X, Y)
        L.data


def test_coxph():

    X = np.random.standard_normal((100,5))
    T = np.random.standard_exponential(100)
    S = np.random.binomial(1, 0.5, size=(100,))

    for case_weights in [np.ones(100), None]:
        L = glm.glm.cox(X, T, S, case_weights=case_weights)
        L.smooth_objective(np.zeros(L.shape), 'both')
        L.saturated_loss.hessian_mult(np.zeros(T.shape), np.ones(T.shape))
        L.hessian(np.zeros(L.shape))

        L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works

        L.gradient(np.zeros(L.shape))

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.data = (X, np.array([T, S]).T)
        L.data


def test_stacked():

    X = np.random.standard_normal((10,5))
    Y = np.random.standard_normal((10,3))

    Xstack = block_columns([X for _ in range(3)])
    for case_weights in [np.ones(30), None]:
        sat = glm.stacked_loglike.gaussian(Y.T)
        L = glm.glm(Xstack, sat.data, sat, case_weights=case_weights)
        L.smooth_objective(np.zeros(L.shape), 'both')

        L.saturated_loss.subsample(np.arange(5)) # check that subsample of saturated loss at least works

        np.testing.assert_allclose(L.gradient(np.zeros(L.shape)),
                                   -X.T.dot(Y))

        Lcp = copy(L)

        L.objective(np.zeros(L.shape))
        L.latexify()

        L.saturated_loss.data

        L.data





"""
Test that GLM agrees with R's glm
"""

from __future__ import print_function

import numpy as np
import nose.tools as nt

try:
    import rpy2.robjects as rpy
    rpy2_available = True
except ImportError:
    rpy2_available = False

import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test(10)
@np.testing.dec.skipif(not rpy2_available, msg="rpy2 not available, skipping test")
def test_gaussian():

    rpy.r('set.seed(0)')
    rpy.r('X = matrix(rnorm(1000), 50, 20)')
    rpy.r('Y = rnorm(50)')
    rpy.r('C = coef(lm(Y~X-1))')
    C = np.asarray(rpy.r('C'))
    X = np.asarray(rpy.r('X'))
    Y = np.asarray(rpy.r('Y'))
    L = rr.glm.gaussian(X, Y)
    soln = L.solve(min_its=200)

    np.testing.assert_allclose(C, soln)

@set_seed_for_test(10)
@np.testing.dec.skipif(not rpy2_available, msg="rpy2 not available, skipping test")
def test_logistic():

    rpy.r('set.seed(0)')
    rpy.r('X = matrix(rnorm(1000), 50, 20)')
    rpy.r('TR = c(rep(1,20), rep(2,30))')
    rpy.r('Y = rbinom(50, TR, 0.5)')
    rpy.r('C = coef(glm(Y/TR~X-1, family=binomial(), weights=TR))')
    C = np.asarray(rpy.r('C'))
    X = np.asarray(rpy.r('X'))
    Y = np.asarray(rpy.r('Y'))
    TR = np.asarray(rpy.r('TR'))
    L = rr.glm.logistic(X, Y, trials=TR)
    soln = L.solve(min_its=200)

    np.testing.assert_allclose(C, soln, atol=1.e-5, rtol=1.e-5)

@set_seed_for_test(10)
@np.testing.dec.skipif(not rpy2_available, msg="rpy2 not available, skipping test")
def test_poisson():

    rpy.r('set.seed(0)')
    rpy.r('X = matrix(rnorm(1000), 50, 20)')
    rpy.r('Y = rpois(50, 5)')
    rpy.r('C = coef(glm(Y~X-1, family=poisson()))')
    C = np.asarray(rpy.r('C'))
    X = np.asarray(rpy.r('X'))
    Y = np.asarray(rpy.r('Y'))
    L = rr.glm.poisson(X, Y)
    soln = L.solve(min_its=200)

    # tolerance adjusted here because of an occasional failure
    # https://nipy.bic.berkeley.edu/builders/regreg-py2.6/builds/30/steps/shell_6/logs/stdio
    np.testing.assert_allclose(C, soln, atol=1.e-5, rtol=1.e-5)

@set_seed_for_test(10)
@np.testing.dec.skipif(not rpy2_available, msg="rpy2 not available, skipping test")
def test_coxph():

    rpy.r('''
    set.seed(0)
    sigma=1.
    X = matrix(rnorm(250), 50, 5)
    X = scale(X, TRUE, TRUE)
    beta = c(3,2,rep(0,3))
    tim = as.vector(X%*%beta + sigma*rnorm(50))
    tim = tim-min(tim)+1
    status = sample(c(0,1),size=50,replace=T)

    library(survival)
    C = coef(coxph(Surv(tim, status) ~ X))
    ''')
    C = np.asarray(rpy.r('C'))
    X = np.asarray(rpy.r('X'))
    T = np.asarray(rpy.r('tim'))
    S = np.asarray(rpy.r('status'))
    L = rr.glm.cox(X, T, S)
    soln = L.solve(min_its=200)

    np.testing.assert_allclose(C, soln, rtol=1.e-4, atol=1.e-4)



from __future__ import print_function, division, absolute_import

import itertools

import numpy as np

import regreg.api as rr
import regreg.atoms.weighted_atoms as WA
from regreg.tests.decorators import set_seed_for_test

from numpy import testing as npt

@set_seed_for_test()
def test_group_lasso_weightedl1_lagrange():
    n, p = 100, 50

    X = np.random.standard_normal((n, p))
    Y = np.random.standard_normal(n)

    loss = rr.glm.gaussian(X, Y)
    weights = np.ones(p)
    weights[-2:] = np.inf
    weights[:2] = 0 
    weight_dict = dict([(i, w) for i, w in enumerate(weights)])
    pen1 = rr.weighted_l1norm(weights, lagrange=0.5 * np.sqrt(n))
    pen2 = rr.group_lasso(np.arange(p), weights=weight_dict, lagrange=0.5 * np.sqrt(n))

    problem1 = rr.simple_problem(loss, pen1)
    problem2 = rr.simple_problem(loss, pen2)

    beta1 = problem1.solve(tol=1.e-14, min_its=500)
    beta2 = problem2.solve(tol=1e-14, min_its=500)

    npt.assert_allclose(beta1, beta2)

    bound_val = pen1.seminorm(beta1, lagrange=1)
    bound1 = rr.weighted_l1norm(weights, bound=bound_val)
    bound2 = rr.group_lasso(np.arange(p), weights=weight_dict, bound=bound_val)
    problem3 = rr.simple_problem(loss, bound1)
    problem4 = rr.simple_problem(loss, bound2)

    beta3 = problem3.solve(tol=1.e-14, min_its=500)
    beta4 = problem4.solve(tol=1.e-14, min_its=500)

    npt.assert_allclose(beta3, beta4)
    npt.assert_allclose(beta3, beta1)

@set_seed_for_test()
def test_group_lasso_weightedl1_bound():
    n, p = 100, 50

    X = np.random.standard_normal((n, p))
    Y = np.random.standard_normal(n)

    loss = rr.glm.gaussian(X, Y)
    weights = np.ones(p)
    weights[-2:] = np.inf
    weights[:2] = 0 
    weight_dict = dict([(i, w) for i, w in enumerate(weights)])
    bound1 = rr.weighted_l1norm(weights, bound=2)
    bound2 = rr.group_lasso(np.arange(p), weights=weight_dict, bound=2)

    problem1 = rr.simple_problem(loss, bound1)
    problem2 = rr.simple_problem(loss, bound2)

    beta1 = problem1.solve(tol=1.e-14, min_its=500)
    beta2 = problem2.solve(tol=1e-14, min_its=500)

    npt.assert_allclose(beta1, beta2)





import numpy as np

have_sklearn = True
try:
    from .base import (sklearn_regression,
                       sklearn_regression_lagrange)
except ImportError:
    have_sklearn = False

from ..smooth.glm import (glm, 
                          huber_svm)

if have_sklearn:
    class sklearn_huber_svm(sklearn_regression):

        def __init__(self, 
                     smoothing_parameter,
                     atom_constructor, 
                     atom_params, 
                     case_weights=False,
                     offset=False,
                     coef=1., 
                     score_method='deviance'):

            """

            Parameters
            ----------

            smoothing_parameter : float
                Smoothing parameter for Huber SVM loss.

            atom_constructor : callable
                Atom constructor that is to to be used as a regularizer.
                Final atom will be constructed as `atom_constructor(**atom_params)`.

            atom_params : dict
                Dictionary of arguments to construct atom.

            offset : bool
                While `y` include an offset column when fitting?

            case_weights : bool
                Will `y` include a case weight column when fitting?

            coef : float
                Scaling to be put in front of loss.

            score_method : str
                Which score to use as default `score`?
                One of ['deviance', 'mean_deviance', 'R2']

            Notes
            -----

            If `case_weights` or `offset` is True, then the `fit`
            method should pass a 2-d array for `y` rather than just a
            response. If only one of these is not None, then the
            quantity is assumed to be the last column.  If both are
            not None, then the 2nd to last are to be the case weights
            and the last is to be the offset.

            The `offset` argument, if not None
            is *subtracted* from the linear predictor
            before evaluating the loss -- this sign is different
            than behavior in e.g. `R` where it is *added* instead.

            """

            self.smoothing_parameter = smoothing_parameter
            sklearn_regression.__init__(self,
                                        atom_constructor,
                                        atom_params,
                                        case_weights=case_weights,
                                        coef=coef,
                                        offset=offset,
                                        score_method=score_method)

        def _loglike_factory(self, X, y):
            response, case_weights_, offset_ = self._check_y_arg(y)
            return glm.huber_svm(X, 
                                 response, 
                                 self.smoothing_parameter,
                                 case_weights=case_weights_,
                                 coef=self.coef,
                                 saturated_offset=offset_)

        def _saturated_score(self,
                             predictions,
                             response,
                             case_weights=None):

            loss = lambda yhat: huber_svm(response.shape,
                                          response,
                                          self.smoothing_parameter,
                                          case_weights=case_weights).smooth_objective(yhat, 'func')

            # negative sign is to align with sklearn's maximizing a score with grid search

            if self.score_method == 'deviance':
                return -loss(predictions)
            elif self.score_method == 'mean_deviance':
                return -loss(predictions) / predictions.shape[0]
            elif self.score_method == 'R2':
                SSE = loss(predictions)
                SST = loss(response.mean() * np.ones_like(response)) # X: right for huber_svm?
                return 1 - SSE / SST
            else:
                return np.nan

    class sklearn_huber_svm_lagrange(sklearn_regression_lagrange, sklearn_huber_svm):

        def __init__(self, 
                     smoothing_parameter,
                     atom_constructor, 
                     atom_params, 
                     case_weights=False,
                     offset=False,
                     enet_alpha=0.,
                     coef=1., 
                     score_method='deviance'):

            """

            Parameters
            ----------

            smoothing_parameter : float
                Smoothing parameter for Huber SVM loss.

            atom_constructor : callable
                Atom constructor that is to to be used as a regularizer.
                Final atom will be constructed as `atom_constructor(**atom_params)`.

            atom_params : dict
                Dictionary of arguments to construct atom.

            offset : bool
                While `y` include an offset column when fitting?

            case_weights : bool
                Will `y` include a case weight column when fitting?

            enet_alpha : float
                Weighting in [0, 1] between ridge and regularizer.

            coef : float
                Scaling to be put in front of loss.

            score_method : str
                Which score to use as default `score`?
                One of ['deviance', 'mean_deviance', 'R2']

            Notes
            -----

            If `case_weights` or `offset` is True, then the `fit`
            method should pass a 2-d array for `y` rather than just a
            response. If only one of these is not None, then the
            quantity is assumed to be the last column.  If both are
            not None, then the 2nd to last are to be the case weights
            and the last is to be the offset.

            The `offset` argument, if not None
            is *subtracted* from the linear predictor
            before evaluating the loss -- this sign is different
            than behavior in e.g. `R` where it is *added* instead.

            """

            self.smoothing_parameter = smoothing_parameter
            sklearn_regression_lagrange.__init__(self,
                                                 atom_constructor,
                                                 atom_params,
                                                 enet_alpha=enet_alpha,
                                                 case_weights=case_weights,
                                                 coef=coef,
                                                 offset=offset,
                                                 score_method=score_method)




"""
Solving a LASSO with linear constraints using NESTA
"""
from __future__ import print_function, division, absolute_import

import numpy as np
import nose.tools as nt

import regreg.api as rr
from regreg.tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_nesta_nonnegative():

    state = np.random.get_state()
    np.random.seed(10)
    n, p, q = 1000, 20, 5
    X = np.random.standard_normal((n, p))
    A = np.random.standard_normal((q,p))

    coef = 10 * np.fabs(np.random.standard_normal(q)) + 1
    coef[:2] = -0.2
    beta = np.dot(np.linalg.pinv(A), coef)
    print(r'\beta', beta)
    print(r'A\beta', np.dot(A, beta))

    Y = np.random.standard_normal(n) + np.dot(X, beta)

    loss = rr.squared_error(X,Y)
    penalty = rr.l1norm(p, lagrange=0.2)
    constraint = rr.nonnegative.linear(A)

    primal, dual = rr.nesta(loss, penalty, constraint, max_iters=300, coef_tol=1.e-10, tol=1.e-10)

    print(r'A \hat{\beta}', np.dot(A, primal))
    assert_almost_nonnegative(np.dot(A,primal), tol=1.e-3)

    np.random.set_state(state)

@set_seed_for_test()
def test_nesta_lasso():

    n, p = 1000, 20
    X = np.random.standard_normal((n, p))
    beta = np.zeros(p)
    beta[:4] = 30
    Y = np.random.standard_normal(n) + np.dot(X, beta)

    loss = rr.squared_error(X,Y)
    penalty = rr.l1norm(p, lagrange=2.)

    # using nesta
    z = rr.zero(p)
    primal, dual = rr.nesta(loss, z, penalty, tol=1.e-10,
                            epsilon=2.**(-np.arange(30)),
                            initial_dual=np.zeros(p))

    # using simple problem

    problem = rr.simple_problem(loss, penalty)
    problem.solve()
    nt.assert_true(np.linalg.norm(primal - problem.coefs) / np.linalg.norm(problem.coefs) < 1.e-3)

    # test None as smooth_atom

    rr.nesta(None, z, penalty, tol=1.e-10,
             epsilon=2.**(-np.arange(30)),
             initial_dual=np.zeros(p))

    # using coefficients to stop

    rr.nesta(loss, z, penalty, tol=1.e-10,
             epsilon=2.**(-np.arange(30)),
             initial_dual=np.zeros(p),
             coef_stop=True)


def assert_almost_nonnegative(b, tol=1.e-6):
    nt.assert_true(np.linalg.norm(b[b<0]) <= tol * np.linalg.norm(b))




import numpy as np
from scipy import sparse
from ..affine import affine_transform

def formD_smaller(m, n):
    """
    Generate a sparse
    matrix that computes
    the difference across all edges in a
    2D lattice of shape (m,n).

    Parameters
    ----------

    m: int

    n: int

    Returns
    -------

    D: scipy.csr_matrix
       A matrix with 2*(m-1)*(n-1)+(m-1)+(n-1) rows
       and m*n columns
       representing each edge exactly once in the
       2D lattice.

       The first 2*(m-1)*(n-1) rows represent
       interior vertices, then the right hand edges and
       finally the top edges with the origin in the bottom left.
    
    """
    p = m*n

    # interior vertices that have
    # both a vertex to their right and above them
    # (starting from bottom left)

    vertices = np.indices((m-1,n-1)).reshape((2, -1)).T
    strides = np.empty((m,n), np.bool).strides
    # D = sparse.lil_matrix((2*(m-1)*(n-1)+(m-1)+(n-1),p))
    
    idx = 0
    vertices_in_strides = np.dot(vertices, strides)
    vertices_in_strides_shift1 = vertices_in_strides + np.dot([1,0], strides)
    vertices_in_strides_shift2 = vertices_in_strides + np.dot([0,1], strides)

    indices = np.array([np.arange(vertices.shape[0]) + idx,
                        vertices_in_strides])

    D1 = sparse.csr_matrix((np.ones(vertices.shape[0]),
                            indices), (2*(m-1)*(n-1)+(m-1)+(n-1),p))
    indices[1] = vertices_in_strides_shift1
    D2 = sparse.csr_matrix((-np.ones(vertices.shape[0]),
                            indices), (2*(m-1)*(n-1)+(m-1)+(n-1),p))
    idx += vertices.shape[0]

    indices = np.array([np.arange(vertices.shape[0]) + idx,
                        vertices_in_strides])
    D3 = sparse.csr_matrix((np.ones(vertices.shape[0]),
                            indices), (2*(m-1)*(n-1)+(m-1)+(n-1),p))
    indices[1] = vertices_in_strides_shift2
    D4 = sparse.csr_matrix((-np.ones(vertices.shape[0]),
                            indices), (2*(m-1)*(n-1)+(m-1)+(n-1),p))

    idx += vertices.shape[0]

    # Now, do the top edges for each
    # vertex in the right edge

    vertices = np.indices((1,n-1)).reshape((2, -1)).T + np.array([m-1,0])
    vertices_in_strides = np.dot(vertices, strides)
    vertices_in_strides_shift3 = vertices_in_strides + np.dot([0,1], strides)

    indices = np.array([np.arange(vertices.shape[0]) + idx,
                        vertices_in_strides])
    D5 = sparse.csr_matrix((np.ones(vertices.shape[0]),
                            indices), (2*(m-1)*(n-1)+(m-1)+(n-1),p))
    indices[1] = vertices_in_strides_shift3
    D6 = sparse.csr_matrix((-np.ones(vertices.shape[0]),
                            indices), (2*(m-1)*(n-1)+(m-1)+(n-1),p))
    idx += vertices.shape[0]

    # Finally, the top edge
    
    vertices = np.indices((m-1,1)).reshape((2, -1)).T + np.array([0,n-1])
    vertices_in_strides = np.dot(vertices, strides)
    vertices_in_strides_shift4 = vertices_in_strides + np.dot([1,0], strides)

    indices = np.array([np.arange(vertices.shape[0]) + idx,
                        vertices_in_strides])
    D7 = sparse.csr_matrix((np.ones(vertices.shape[0]),
                            indices), (2*(m-1)*(n-1)+(m-1)+(n-1),p))
    indices[1] = vertices_in_strides_shift4
    D8 = sparse.csr_matrix((-np.ones(vertices.shape[0]),
                            indices), (2*(m-1)*(n-1)+(m-1)+(n-1),p))

    return D1 + D2 + D3 + D4 + D5 + D6 + D7 + D8


def formD(m, n):
    """
    Generate a sparse
    matrix that computes
    the difference across all edges in a
    2D lattice of shape (m,n).

    Parameters
    ----------
    m: int

    n: int

    Returns
    -------

    D: scipy.csr_matrix
       A matrix with 2*(m*n-1)
       and m*n columns
       representing each edge exactly once in the
       2D lattice and including some zero rows.

       This shape is used because it associates
       exactly two edges to each vertex except
       the top right. Hence, the differences
       can be formed into a matrix with two rows
       to which the l1_l2 proximal function
       can be applied.

       The first (m-1)*(n-1) rows represent
       interior vertices, then the right hand edges and
       finally the top edges with the origin in the bottom left.
       This forms the first row of this two row matrix described above.

       The second row of the matrix has exactly the same structure.

    """

    p = m*n

    # interior vertices that have
    # both a vertex to their right and above them
    # (starting from bottom left)

    vertices = np.indices((m-1,n-1)).reshape((2, -1)).T
    strides = np.empty((m,n), np.bool).strides
    nrow = 2*(m*n-1)
    
    idx = 0
    vertices_in_strides = np.dot(vertices, strides)
    vertices_in_strides_shift1 = vertices_in_strides + np.dot([1,0], strides)

    indices = np.array([np.arange(vertices.shape[0]) + idx,
                        vertices_in_strides])

    D1 = sparse.csr_matrix((np.ones(vertices.shape[0]),
                            indices), (nrow,p))
    indices[1] = vertices_in_strides_shift1
    D2 = sparse.csr_matrix((-np.ones(vertices.shape[0]),
                            indices), (nrow,p))
    idx += vertices.shape[0]

    # now, do the top edge

    vertices = np.indices((1,n-1)).reshape((2, -1)).T + np.array([m-1,0])
    vertices_in_strides = np.dot(vertices, strides)
    vertices_in_strides_shift3 = vertices_in_strides + np.dot([0,1], strides)

    indices = np.array([np.arange(vertices.shape[0]) + idx,
                        vertices_in_strides])
    D5 = sparse.csr_matrix((np.ones(vertices.shape[0]),
                            indices), (nrow,p))
    indices[1] = vertices_in_strides_shift3
    D6 = sparse.csr_matrix((-np.ones(vertices.shape[0]),
                            indices), (nrow,p))
    idx += vertices.shape[0] 

    # Finally, the top edge
    
    vertices = np.indices((m-1,1)).reshape((2, -1)).T + np.array([0,n-1])
    vertices_in_strides = np.dot(vertices, strides)
    vertices_in_strides_shift4 = vertices_in_strides + np.dot([1,0], strides)

    indices = np.array([np.arange(vertices.shape[0]) + idx,
                        vertices_in_strides])
    D7 = sparse.csr_matrix((np.ones(vertices.shape[0]),
                            indices), (nrow,p))
    indices[1] = vertices_in_strides_shift4
    D8 = sparse.csr_matrix((-np.ones(vertices.shape[0]),
                            indices), (nrow,p))
    idx += vertices.shape[0]

    # this is the second 'row' of the tensor, which is
    # nonzero only for interior vertices

    vertices = np.indices((m-1,n-1)).reshape((2, -1)).T
    vertices_in_strides = np.dot(vertices, strides)
    vertices_in_strides_shift2 = vertices_in_strides + np.dot([0,1], strides)

    indices = np.array([np.arange(vertices.shape[0]) + idx,
                        vertices_in_strides])
    D3 = sparse.csr_matrix((np.ones(vertices.shape[0]),
                            indices), (nrow,p))
    indices[1] = vertices_in_strides_shift2
    D4 = sparse.csr_matrix((-np.ones(vertices.shape[0]),
                            indices), (nrow,p))


    return D1 + D2 + D3 + D4 + D5 + D6 + D7 + D8

class image2d_differences(affine_transform):

    def __init__(self, image_shape, affine_offset=None):
        self.image_shape = image_shape
        self.D = formD(*image_shape)
        self.DT = self.D.T.tocsr()
        self.input_shape = image_shape
        m, n = self.input_shape
        self.output_shape = (m*n-1,2)
        self.affine_offset = affine_offset

    def linear_map(self, x, copy=True):
        r"""Apply linear part of transform to `x`

        Return :math:`Dx`

        Parameters
        ----------
        x : ndarray
            array to which to apply transform.  Can be 1D or 2D
        copy : {True, False}, optional
            If True, in situations where return is identical to `x`, ensure
            returned value is a copy.

        Returns
        -------
        Dx : ndarray
            `x` transformed with linear component
        """
        v = self.D * x.reshape(-1)
        return v.reshape(self.output_shape[::-1]).T

    def affine_map(self, x, copy=True):
        r"""Apply linear part of transform to `x`

        Return :math:`Dx+\alpha`

        Parameters
        ----------
        x : ndarray
            array to which to apply transform.  Can be 1D or 2D
        copy : {True, False}, optional
            If True, in situations where return is identical to `x`, ensure
            returned value is a copy.

        Returns
        -------
        Dx + offset: ndarray
            `x` transformed with linear component + offset
        """
        if self.affine_offset is not None:
            return self.linear_map(x) + self.affine_offset
        return self.linear_map(x)

    def adjoint_map(self, u, copy=True):
        r"""Apply adjioint of transform to `u`

        Return :math:`D^Tu`

        Parameters
        ----------
        u : ndarray
            array to which to apply transform.  Can be 1D or 2D
        copy : {True, False}, optional
            If True, in situations where return is identical to ``, ensure
            returned value is a copy.

        Returns
        -------
        D.T*u : ndarray
            `u` transformed with linear component
        """
        v = self.DT * u.T.reshape(-1)
        return v.reshape(self.input_shape)



"""
This module has a class for specifying a problem from just
a smooth function and a single penalty.
"""
from __future__ import print_function, division, absolute_import

from functools import partial

import numpy as np, warnings
from scipy.optimize import fmin_powell

from .composite import composite
from .simple import simple_problem
from ..affine import identity, scalar_multiply, astransform, adjoint
from ..atoms import atom
from ..smooth import sum as smooth_sum, affine_smooth
from ..smooth.quadratic import quadratic_loss
from ..identity_quadratic import identity_quadratic
from ..algorithms import FISTA

class quasi_newton(object):
    
    """
    Minimize an objective using proximal Newton algorithm
    as described [here](https://arxiv.org/pdf/1206.1623.pdf).

    Uses BFGS updates.

    """

    damping_term = 1.

    def __init__(self, 
                 smooth_atom, 
                 proximal_atom,
                 initial_hessian,
                 initial_step=1.,
                 quadratic=None, # e.g. for ridge 
                 ):
        '''
        Parameters
        ----------

        smooth_objective : `smooth_atom`
            Smooth objective function.

        proximal_map : `atom`
            Nonsmooth part -- its proximal map is used in solving
            inner loop.

        initial_hessian : ndarray
            Guess at initial Hessian.

        initial_step : float
            Guess at initial stepsize.

        quadratic : `identity_quadratic`
            Quadratic term optionally
            added to overall objective.

        '''
        self.smooth_atom = smooth_atom
        self.proximal_atom = proximal_atom
        self.coefs = self.smooth_atom.coefs = self.proximal_atom.coefs
        H0 = initial_hessian
        self.hessian = H0 + (self.damping_term *
                             (np.diag(H0).mean() *
                              np.identity(H0.shape[0])))
        self.quadratic_loss = quadratic_loss(self.coefs.shape,
                                             Q=self.hessian)
        if quadratic is None:
            quadratic = identity_quadratic(0, 0, 0, 0)
        self.quadratic = quadratic

        self.quadratic_problem = simple_problem(self.quadratic_loss,
                                                proximal_atom)
        self.stepsize = initial_step

    def objective(self,
                  coefs):
        """
        Total objective

        Parameters
        ----------

        coefs : np.ndarray
            Where to evaluate the objective

        """
        return (self.smooth_objective(coefs, 'func') +
                self.nonsmooth_objective(coefs, 'func'))

    def update_hessian(self,
                       old_soln,
                       new_soln,
                       old_grad,
                       new_grad):
        """
        Update current Hessian using 
        [BFGS](https://en.wikipedia.org/wiki/Broyden%E2%80%93Fletcher%E2%80%93Goldfarb%E2%80%93Shanno_algorithm)
        updating rule.

        Parameters
        ----------

        old_soln : ndarray
            Current position.

        new_soln : ndarray
            Next position.

        old_grad : ndarray
            Current gradient.

        new_grad : ndarray
            Next gradient.

        """
        # update the hessian, 
        # updates self.quadratic_loss in place

        # uses BFGS

        y = new_grad - old_grad
        s = new_soln - old_soln

        Bs = self.hessian.dot(s)

        if (y.dot(s) < -1e-5):
            raise ValueError('failure in BFGS update -- dot product is negative')

        # update is done in place
        # note that this means self.quadratic_problem.Q is also updated!
        
        self.hessian += np.multiply.outer(y, y) / y.dot(s) - np.multiply.outer(Bs, Bs) / s.dot(Bs)


    def smooth_objective(self, x, mode='both', check_feasibility=False):
        '''
        Compute value and / or gradient of our smooth part.

        Parameters
        ----------

        x : ndarray
            Argument to smooth objective.

        Returns
        -------

        f : float (optional)
            Value of smooth objective at `x`.

        g : ndarray (optional)
            Gradient of smooth objective at `x`.

        '''
        vs = self.smooth_atom.smooth_objective(x, mode, check_feasibility)
        return vs

    def nonsmooth_objective(self, x, check_feasibility=False):
        '''
        Compute value and / or gradient of nonsmooth part.

        Parameters
        ----------

        x : ndarray
            Argument to nonsmooth objective.

        Returns
        -------

        f : float (optional)
            Value of nonsmooth objective at `x`.

        '''
        vn = self.proximal_atom.nonsmooth_objective(x, check_feasibility=check_feasibility)
        vs = self.smooth_atom.nonsmooth_objective(x, check_feasibility=check_feasibility)
        return vn + vs + self.quadratic.objective(x, 'func')

    def proximal(self, proxq):
        proxq = proxq + self.smooth_atom.quadratic + self.quadratic
        return self.proximal_atom.solve(proxq)

    def latexify(self, var=None):
        template_dict = self.objective_vars.copy()
        template_dict['prox'] =  self.proximal_atom.latexify(var=var,idx='2')
        template_dict['smooth'] = self.smooth_atom.latexify(var=var,idx='1')
        result = r'''
        \text{BFGS Quasi-Newton for}
        \begin{aligned}
        \text{minimize}_{%(var)s} & f(%(var)s) + g(%(var)s) \\
        f(%(var)s) &= %(smooth)s \\
        g(%(var)s) &= %(prox)s \\
        \end{aligned}
        ''' % template_dict
        result = '\n'.join([s.strip() for s in result.split('\n')])
        return result

    def solve_inner_loop(self, 
                         return_optimum=False, 
                         initial_step=None,
                         **fit_args):
        
        '''
        Inner loop of quasi-Newton.
        Given `x, G, H` our current position, gradient and
        Hessian of the smooth part we minimize
        (quasi) 2nd order Taylor approximation of
        smooth part plus the nonsmooth atom.

        .. math::

            \Delta \mapsto G^T\Delta + \frac{1}{2} \Delta^TH\Delta + {\cal P}(\Delta + x)

        Parameters
        ----------

        niter : int
            Number of quasi-Newton steps to take.

        Returns
        -------

        coefs : ndarray
            Optimal solution.
          
        '''

        # return search direction 
        # loss is 
        # x -> (\nabla g(x_cur) - Q x_cur)^Tx + 1/2 x^TQx + P(x)
        #
        # this gives a direction x^* - x_cur (with x^* optimizer from above)

        Q = self.quadratic_loss.Q
        grad = self.smooth_objective(self.coefs,
                                     'grad')
        linear_term = (grad - Q.dot(self.coefs))
        quad = identity_quadratic(0, 0, linear_term, 0)

        # warm start the subproblem
        self.quadratic_problem.coefs[:] = self.coefs 

        quad_soln = self.quadratic_problem.solve(quad + self.quadratic,
                                                 **fit_args)
        direction = quad_soln - self.coefs
        return direction, grad, quad_soln 

    def solve(self,
              niter=30,
              maxfun=10,
              maxiter=5,
              **fit_args # for inner loop solver
              ):
        
        '''
        Run quasi-Newton for `niter` steps.

        Parameters
        ----------

        niter : int
            Number of quasi-Newton steps to take.

        maxfun : int
            Maximum function evaluations passed to `fmin_powell`

        maxiter : int
            Maximum iterations passed to `fmin_powell`

        Returns
        -------

        coefs : ndarray
            Optimal solution.
          
        '''

        self.solver_results = [np.inf]
        for i in range(niter):
            direction, grad, quad_soln = self.solve_inner_loop(**fit_args)
            fit_args['start_step'] = 1.5 * self.quadratic_problem.final_step
            stepsize, val = self.backtrack(direction,
                                           maxfun=maxfun,
                                           maxiter=maxiter)
            delta = stepsize * direction
            new_grad = self.smooth_objective(self.coefs + delta, 'grad')

            if self.converged(self.solver_results[-1],
                              val,
                              self.coefs,
                              self.coefs + delta,
                              grad,
                              new_grad):
                break

            self.update_hessian(self.coefs, self.coefs + delta, grad, new_grad)

            # update state 

            self.coefs = self.coefs + delta
            self.stepsize = stepsize
            self.solver_results.append(val)

        self.solver_results = np.array(self.solver_results)
        self.coefs[:] = quad_soln # ensures the last solution
                                  # will satisfy the constraint
                                  # if in bound form
        return self.coefs

    def converged(self,
                  old_value,
                  new_value,
                  old_soln,
                  new_soln,
                  old_grad,
                  new_grad):
        """
        Parameters
        ----------

        old_soln : ndarray
            Current value.

        new_soln : ndarray
            Next value.

        old_soln : ndarray
            Current position.

        new_soln : ndarray
            Next position.

        old_grad : ndarray
            Current gradient.

        new_grad : ndarray
            Next gradient.
        """
        return (np.fabs(old_soln - new_soln).max()
                < 1.e-6 * np.linalg.norm(new_soln))

    def backtrack(self, 
                  direction,
                  maxiter=10,
                  maxfun=6):
        '''
        Run a simple backtracking using `scipy.optimize.fmin_powell`
        to determine optimal stepsize.

        Parameters
        ----------

        direction : ndarray
            Direction for backtracking search.
            Found by solving inner optimization problem
            in quasi-Newton algorithm.

        maxiter : int
            Maximum number of iterations in `fmin_powell`

        Returns
        -------

        stepsize : float
            Optimal stepsize.
          
        '''

        BIG = 1e16

        def restriction(direction, coefs, stepsize):
            val = (self.smooth_objective(coefs + stepsize * direction, 'func') +
                   self.nonsmooth_objective(coefs + stepsize * direction,
                                            check_feasibility=True))
            if np.isnan(val):
                val = BIG
            grad = self.smooth_objective(coefs + stepsize * direction, 'grad')
            if np.any(np.isnan(grad)):
                val += BIG
            return val

        restriction = partial(restriction, direction, self.coefs)
        step =  fmin_powell(restriction,
                            1, 
                            disp=False,
                            maxiter=maxiter,
                            maxfun=maxfun)
        return step, restriction(step)



import numpy as np

import regreg.api as rr 
import regreg.affine as ra
from regreg.problems.admm import admm_problem

def test_admm(n=100, p=10):

    X = np.random.standard_normal((n, p))
    Y = np.random.standard_normal(n)
    loss = rr.squared_error(X, Y)
    D = np.identity(p)
    pen = rr.l1norm(p, lagrange=1.5)

    ADMM = admm_problem(loss, pen, ra.astransform(D), 0.5)
    ADMM.solve(niter=1000)

    coef1 = ADMM.atom_coefs
    problem2 = rr.simple_problem(loss, pen)
    coef2 = problem2.solve(tol=1.e-12, min_its=500)

    np.testing.assert_allclose(coef1, coef2, rtol=1.e-3, atol=1.e-4)
    



from numpy import zeros, asarray, any as npany
from copy import copy

# local imports

from ..identity_quadratic import identity_quadratic as sq
from ..algorithms import FISTA

from ..objdoctemplates import objective_doc_templater
from ..doctemplates import (doc_template_user, doc_template_provider)

@objective_doc_templater()
class composite(object):
    """
    A generic way to specify a problem in composite form.
    """

    objective_template = r"""f(%(var)s)"""
    objective_vars = {'var': r'\beta', 'shape':'p', 'offset':r'\alpha'}

    def __init__(self, shape, offset=None,
                 quadratic=None, initial=None):

        self.offset = offset

        if type(shape) == type(1):
            self.shape = (shape,)
        else:
            self.shape = shape

        if quadratic is not None:
            self.quadratic = quadratic
        else:
            self.quadratic = sq(0,0,0,0)

        if initial is None:
            self.coefs = zeros(self.shape)
        else:
            self.coefs = copy(initial)

    def set_offset(self, value):
        if value is not None:
            value = asarray(value)
            if npany(value != 0):
                self._offset = value
        else:
            self._offset = None

    def get_offset(self):
        if not hasattr(self, "_offset"):
            self._offset = None
        return self._offset
    offset = property(get_offset, set_offset)

    def latexify(self, var=None, idx=''):
        template_dict = copy(self.objective_vars)
        template_dict['idx'] = idx
        if var is not None:
            template_dict['var'] = var
        if hasattr(self, 'offset') and self.offset is not None and npany(self.offset != 0):
            template_dict['var'] = (r'%(var)s - %(offset)s_{%(idx)s}' % template_dict)

        obj = self.objective_template % template_dict
        template_dict['obj'] = obj

        if not self.quadratic.iszero:
            return ' + '.join([obj, self.quadratic.latexify(var=var, idx=idx)])
        return obj

    def _repr_latex_(self):
        return r'$$' + self.latexify() + r'$$'

    def nonsmooth_objective(self, x, check_feasibility=False):
        return self.quadratic.objective(x, 'func')

    def smooth_objective(self, x, mode='both', check_feasibility=False):
        '''
        The smooth_objective and the quadratic_objective combined.
        '''
        raise NotImplementedError

    def objective(self, x, check_feasibility=False):
        return self.smooth_objective(x,mode='func', check_feasibility=check_feasibility) + self.nonsmooth_objective(x, check_feasibility=check_feasibility)

    @doc_template_provider
    def proximal_optimum(self, quadratic):
        r"""
        Compute a proximal step, and return the minimizer
        and nonsmooth objective the value.
        """
        argmin = self.proximal(quadratic)
        if self.quadratic is None:
            return argmin, quadratic.objective(argmin) + self.nonsmooth_objective(argmin) 
        else: 
            return argmin, quadratic.objective(argmin) + self.nonsmooth_objective(argmin) + self.quadratic.objective(argmin, 'func') 

    def proximal_step(self, quadratic, prox_control=None):
        """
        Compute the proximal optimization

        Parameters
        ----------
        prox_control: [None, dict]
            If not None, then a dictionary of parameters for the prox procedure
        """
        # This seems like a null op -- if all proximals accept optional prox_control
        if prox_control is None:
            return self.proximal(quadratic)
        else:
            return self.proximal(quadratic, prox_control=prox_control)

    def apply_offset(self, x):
        """
        If self.offset is not None, return x-self.offset, else return x.
        """
        if self.offset is not None:
            return x - self.offset
        return x

    def set_quadratic(self, quadratic):
        """
        Set the quadratic part of the composite.
        """
        self._quadratic = quadratic

    def get_quadratic(self):
        """
        Get the quadratic part of the composite.
        """
        if not hasattr(self, "_quadratic"):
            self._quadratic = sq(None, None, None, None)
        return self._quadratic
    quadratic = property(get_quadratic, set_quadratic, doc='Quadratic part of the object, instance of `regreg.identity_quadratic.identity_quadratic`.')

    def smoothed(self, smoothing_quadratic):
        '''
        Add quadratic smoothing term
        '''
        conjugate_atom = copy(self.conjugate)
        sq = smoothing_quadratic
        if sq.coef in [None, 0]:
            raise ValueError('quadratic term of smoothing_quadratic must be non 0')
        total_q = sq

        if conjugate_atom.quadratic is not None:
            total_q = sq + conjugate_atom.quadratic
        conjugate_atom.set_quadratic(total_q)
        smoothed_atom = conjugate_atom.conjugate
        return smoothed_atom

    def solve(self, quadratic=None, return_optimum=False, **fit_args):
        raise NotImplementedError('subclasses must implement their own solve methods')

@objective_doc_templater()
class nonsmooth(composite):
    """
    A composite subclass that explicitly returns 0
    as smooth_objective.
    """

    def smooth_objective(self, x, mode='both', check_feasibility=False):
        '''
        The zero function.
        '''

        if mode == 'both':
            return 0., zeros(x.shape)
        elif mode == 'func':
            return 0.
        elif mode == 'grad':
            return zeros(x.shape)
        raise ValueError("Mode not specified correctly")

    def solve(self, quadratic=None, return_optimum=False, **fit_args):
        if quadratic is None:
            quadratic = sq(0, 0, 0, 0)
        self.coefs = self.proximal(quadratic)
        if return_optimum:
            return self.objective(self.coefs) + quadratic.objective(self.coefs, 'func'), self.coefs
        else:
            return self.coefs

@objective_doc_templater()
class smooth(composite):

    """
    A composite subclass that has 0 as 
    nonsmooth_objective and the proximal
    is a null-op.
    """

    objective_vars = copy(composite.objective_vars)
    objective_vars['coef'] = 'C'

    def get_lipschitz(self):
        if hasattr(self, '_lipschitz'):
            return self._lipschitz + self.quadratic.coef
        return self.quadratic.coef

    def set_lipschitz(self, value):
        if value < 0:
            raise ValueError('Lipschitz constant must be non-negative')
        self._lipschitz = value
    lipschitz = property(get_lipschitz, set_lipschitz)

    def smooth_objective(self, x, mode='func', check_feasibility=False):
        return self._smooth_objective(x, mode=mode, check_feasibility=check_feasibility)

    def proximal(self, quadratic):
        totalq = self.quadratic + quadratic
        return -totalq.linear_term / totalq.coef

    def solve(self, quadratic=None, return_optimum=False, **fit_args):
        if quadratic is None:
            quadratic = sq(0, 0, 0, 0)
        oldq, self.quadratic = self.quadratic, self.quadratic + quadratic
        self.solver = FISTA(self)
        self.solver.fit(**fit_args)
        self.quadratic = oldq

        if return_optimum:
            return self.objective(self.coefs), self.coefs
        else:
            return self.coefs

class smooth_conjugate(smooth):

    def __init__(self, atom, smoothing_quadratic=None):
        """
        Given an atom,
        compute the conjugate of this atom plus 
        an identity_quadratic which will be 
        a smooth version of the conjugate of the atom.

        should we have an argument "collapse" that makes a copy?

        """
        # this holds a pointer to the original atom,
        # but will be replaced later

        self.atom = atom
        if smoothing_quadratic is None:
            smoothing_quadratic = sq(0,0,0,0)
        self.smoothing_quadratic = smoothing_quadratic
        total_quadratic = self.atom.quadratic + self.smoothing_quadratic

        if total_quadratic.coef in [0,None]:
            raise ValueError('the atom must have non-zero quadratic term to compute ensure smooth conjugate')

        self.lipschitz = 1. / total_quadratic.coef

        self.shape = atom.shape
        self.coefs = zeros(self.shape)

        # A smooth conjugate is the conjugate of some $f$ with an identity quadratic added to it, or
        # $$
        # h(u) = \sup_x \left( u^Tx - \frac{\kappa}{2} \|x\|^2_2 - \beta^Tx-c-f(x) \right).
        # $$
        # Suppose we add a quadratic to $h$ to get
        # $$
        # \tilde{h}(u) = \frac{r}{2} \|u\|^2_2 + u^T\gamma + a + h(u)$$
        # and take the conjugate again:
        # $$
        # \begin{aligned}
        # g(v) &= \sup_{u} u^Tv - \tilde{h}(u) \\
        # &= \sup_u u^Tv -  \frac{r}{2} \|u\|^2_2 - u^T\gamma-a - h(u) \\
        # &=  \sup_u u^Tv - \frac{r}{2} \|u\|^2_2 - u^T\gamma-a  - \sup_x \left( u^Tx - \frac{\kappa}{2} \|x\|^2_2 - \beta^Tx-c-f(x)  \right)\\
        # &= \sup_u u^Tv - \frac{r}{2} \|u\|^2_2 - u^T\gamma-a + \inf_x \left(  \frac{\kappa}{2} \|x\|^2_2  +\beta^Tx + c +f(x) - u^Tx  \right)\\
        # &= \sup_u \inf_x u^Tv - \frac{r}{2} \|u\|^2_2 - u^T\gamma-a +  \frac{\kappa}{2} \|x\|^2_2 + \beta^Tx + c +f(x) - u^Tx \\
        # &=  \inf_x \sup_u u^Tv - \frac{r}{2} \|u\|^2_2 - u^T\gamma-a +  \frac{\kappa}{2} \|x\|^2_2  + \beta^Tx + c +f(x) - u^Tx \\
        # &=  \inf_x \sup_u \left(u^Tv - \frac{r}{2} \|u\|^2_2 - u^T\gamma- u^Tx\right)-a +  \frac{\kappa}{2} \|x\|^2_2 +   \beta^Tx + c +f(x)  \\
        # &=  \inf_x \frac{1}{2r} \|x+\gamma-v\|^2_2 -a +  \frac{\kappa}{2} \|x\|^2_2 + \beta^Tx + c +f(x)  \\
        # &= c-a + \frac{1}{2r} \|\gamma-v\|^2_2 - \sup_x \left((v/r)^Tx - \left(\frac{1}{r} + \kappa\right) \|x\|^2_2 - x^T(\beta+\gamma/r) - f(x) \right) \\
        # \end{aligned}
        # $$

        # This says that for $r > 0$ the conjugate of a smooth conjugate with a quadratic added to it is a quadratic plus a modified smooth conjugate evaluated at $v/r$.

        # What if $r=0$? Well,
        # then 
        # $$
        # \begin{aligned}
        # g(v) &= \sup_{u} u^Tv - \tilde{h}(u) \\
        # &= \sup_u u^Tv  - u^T\gamma-a - h(u) \\
        # &=  \sup_u u^Tv  - u^T\gamma-a  - \sup_x \left( u^Tx - \frac{\kappa}{2} \|x\|^2_2 - \beta^Tx-c-f(x)  \right)\\
        # &= \sup_u u^Tv - u^T\gamma-a + \inf_x \left(  \frac{\kappa}{2} \|x\|^2_2  +\beta^Tx + c +f(x) - u^Tx  \right)\\
        # &= \sup_u \inf_x u^Tv - u^T\gamma-a +  \frac{\kappa}{2} \|x\|^2_2 + \beta^Tx + c +f(x) - u^Tx \\
        # &= \inf_x \sup_u u^Tv - u^T\gamma-a +  \frac{\kappa}{2} \|x\|^2_2 + \beta^Tx + c +f(x) - u^Tx \\
        # &=   \frac{\kappa}{2} \|v-\gamma\|^2_2 + \beta^T(v-\gamma) + c-a +f(v-\gamma) \\
        # \end{aligned}
        # $$
        # where, in the last line we have used the fact that the $\sup$ over $u$ in the second to last line is infinite unless $x=v-\gamma$.

    def get_conjugate(self):
        if self.quadratic.iszero:
            if self.smoothing_quadratic.iszero:
                return self.atom
            else:
                atom = copy(self.atom)
                atom.quadratic = atom.quadratic + self.smoothing_quadratic
                return atom
        else:
            q = self.quadratic.collapsed()
            if q.coef == 0:
                newq = copy(atom.quadratic)
                newq.constant_term -= q.constant_term
                offset = -q.linear_term
                if atom.offset is not None:
                    offset -= atom.offset
                atom = copy(atom)
                atom.offset = offset
                atom.quadratic = newq
                return atom
            if q.coef != 0:
                r = q.coef
                sq_ = self.smoothing_quadratic
                newq = sq_ + q.conjugate
                new_smooth = smooth_conjugate(self.atom, quadratic=newq)
                output = smooth(self.atom.shape,
                                offset=None,
                                quadratic=sq(1./r, q.linear_term, 0, 0),
                                initial=None)
                output.smoothed_atom = new_smooth

                def smooth_objective(self, x, mode='func', check_feasibility=False):
                    # what if self.quadratic is later changed? hmm..
                    r = 1. / self.quadratic.coef
                    if mode == 'func':
                        v = self.smoothed_atom.smooth_objective(x/r, mode=mode, 
                                                                check_feasibility=check_feasibility)
                        return self.smoothing_quadratic.objective(x, 'func') - v
                    elif mode == 'both':
                        v1, g1 = self.smoothed_atom.smooth_objective(x/r, mode=mode, 
                                                                     check_feasibility=check_feasibility)
                        v2, g2 = self.smoothing_quadratic.objective(x, mode=mode, 
                                                                    check_feasibility=check_feasibility)
                        return v2-v1, g2-g1/r
                    elif mode == 'grad':
                        g1 = self.smoothed_atom.smooth_objective(x/r, mode='grad', 
                                                                     check_feasibility=check_feasibility)
                        g2 = self.smoothing_quadratic.objective(x, mode='grad', 
                                                                    check_feasibility=check_feasibility)
                        return g2-g1/r
                    else:
                        raise ValueError("mode incorrectly specified")
                        
                output.smooth_objective = type(output.smooth_objective)(smooth_objective,
                                                                        output, 
                                                                        smooth)
                return output
    conjugate = property(get_conjugate)

    def latexify(self, var=None, idx=''):
        template_dict = copy(self.atom.objective_vars)
        template_dict['idx'] = idx
        if var is not None:
            template_dict['var'] = var

        template_dict['f'] = self.atom.latexify(var='u', idx=idx)
        quad_latex = self.smoothing_quadratic.latexify(var='u', idx=idx)
        if quad_latex:
            template_dict['f'] = ' + '.join([template_dict['f'],
                                             quad_latex])

        objective = r' \sup_{u \in \mathbb{R}^{%(shape)s} } \left[ \langle %(var)s, u \rangle - \left(%(f)s \right) \right]' % template_dict
        return objective

    def __repr__(self):
        return 'smooth_conjugate(%s,%s)' % (repr(self.atom), repr(self.smoothing_quadratic + self.atom.quadratic))

    def smooth_objective(self, x, mode='both', check_feasibility=False):
        """
        Evaluate a smooth function and/or its gradient

        if mode == 'both', return both function value and gradient
        if mode == 'grad', return only the gradient
        if mode == 'func', return only the function value
        """

        q = self.smoothing_quadratic + sq(0,0,-x,0) 

        if mode == 'both':
            optimal_value, argmin = self.atom.solve(quadratic=q, return_optimum=True)
            objective = -optimal_value
            # retain a reference
            self.grad = argmin
            return objective, self.grad
        elif mode == 'grad':
            argmin = self.atom.solve(quadratic=q)
            # retain a reference
            self.grad = argmin
            return self.grad
        elif mode == 'func':
            optimal_value, argmin = self.atom.solve(quadratic=q, return_optimum=True)
            objective = -optimal_value
            # retain a reference
            self.grad = argmin
            return objective
        else:
            raise ValueError("mode incorrectly specified")

    def proximal(self, proxq, prox_control=None):
        raise ValueError('no proximal defined')



from __future__ import print_function, division, absolute_import

import itertools

import numpy as np

import regreg.api as rr
import regreg.atoms.weighted_atoms as WA
from regreg.tests.decorators import set_seed_for_test

from numpy import testing as npt

from .test_seminorms import Solver, SolverFactory

w1 = np.ones(20) * 0.5
w2 = w1 * 0
w2[:10] = 2.

class WeightedSolverFactory(SolverFactory):

    weight_choices = [w1, w2]
    FISTA_choices = [True]
    L_choices = [0.3]
    coef_stop_choices = [False]

    def __init__(self, klass, mode):
        self.klass = klass
        self.mode = mode

    def __iter__(self):
        for (offset,
             FISTA,
             coef_stop,
             L,
             q,
             w) in itertools.product(self.offset_choices,
                                     self.FISTA_choices,
                                     self.coef_stop_choices,
                                     self.L_choices,
                                     self.quadratic_choices,
                                     self.weight_choices):
            self.FISTA = FISTA
            self.coef_stop = coef_stop
            self.L = L

            if self.mode == 'lagrange':
                atom = self.klass(w, lagrange=self.lagrange)
            else:
                atom = self.klass(w, bound=self.bound)

            if q: 
                atom.quadratic = rr.identity_quadratic(0,
                                                       0,
                                                       np.random.standard_normal(atom.shape)*0.02)

            if offset:
                atom.offset = 0.02 * np.random.standard_normal(atom.shape)

            solver = Solver(atom,
                            interactive=self.interactive, 
                            coef_stop=coef_stop,
                            FISTA=FISTA,
                            L=L)
            yield solver

@set_seed_for_test()
@np.testing.dec.slow
def test_proximal_maps():
    for klass, mode in itertools.product([WA.l1norm, WA.supnorm],
                                         ['lagrange', 'bound']):
        factory = WeightedSolverFactory(klass, mode)
        for solver in factory:
            for t in solver.all_tests():
                yield t

@set_seed_for_test()
def test_weighted_l1():
    a =rr.weighted_l1norm(2*np.ones(10), lagrange=0.5)
    b= rr.l1norm(10, lagrange=1)
    z = np.random.standard_normal(10)
    npt.assert_equal(b.lagrange_prox(z), a.lagrange_prox(z))
    npt.assert_equal(b.dual[1].bound_prox(z), a.dual[1].bound_prox(z))

@set_seed_for_test()
def test_weighted_l1_bound():
    a =rr.weighted_supnorm(2*np.ones(10), bound=2)
    b= rr.supnorm(10, bound=1)
    z = np.random.standard_normal(10)
    npt.assert_equal(b.bound_prox(z), a.bound_prox(z))
    npt.assert_equal(b.dual[1].lagrange_prox(z), a.dual[1].lagrange_prox(z))

@set_seed_for_test()
def test_weighted_l1_with_zero():
    z = np.random.standard_normal(5)
    a=rr.weighted_l1norm([0,1,1,1,1], lagrange=0.5)
    b=a.dual[1]
    c=rr.l1norm(4, lagrange=0.5)
    npt.assert_equal(a.lagrange_prox(z), z-b.bound_prox(z))
    npt.assert_equal(a.lagrange_prox(z)[0], z[0])
    npt.assert_equal(a.lagrange_prox(z)[1:], c.lagrange_prox(z[1:]))

@set_seed_for_test()
def test_weighted_l1_bound_loose():
    n, p = 100, 10
    X = np.random.standard_normal((n, p))
    Y = np.random.standard_normal(n)
    beta = np.linalg.pinv(X).dot(Y)
    bound = 2 * np.fabs(beta).sum()
    atom = rr.weighted_l1norm(np.ones(p), bound=bound)
    loss = rr.squared_error(X, Y)
    problem = rr.simple_problem(loss, atom)
    soln = problem.solve(tol=1.e-12, min_its=100)
    npt.assert_allclose(soln, beta)
    



"""
This module contains the implementation operator and nuclear norms, used in 
matrix completion problems and other low-rank factorization
problems.

"""
from __future__ import print_function, division, absolute_import

import numpy as np

from ..affine import affine_sum, astransform, adjoint
from ..atoms.svd_norms import (svd_atom, nuclear_norm as nuclear_norm_atom,
                               operator_norm as operator_norm_atom)
from ..atoms.seminorms import l1norm, _work_out_conjugate
from ..problems.composite import smooth_conjugate

from ..objdoctemplates import objective_doc_templater
from ..doctemplates import doc_template_user

@objective_doc_templater()
class factored_matrix(object):

    """
    A class for storing the SVD of a linear_tranform. 
    Has affine_transform attributes like linear_map.
    """

    def __init__(self,
                 linear_operator,
                 min_singular=0.,
                 tol=1e-5,
                 initial_rank=None,
                 initial = None,
                 affine_offset=None,
                 debug=False):

        self.affine_offset = affine_offset
        self.tol = tol
        self.initial_rank = initial_rank
        self.initial = initial
        self.debug = debug

        if min_singular >= 0:
            self.min_singular = min_singular
        else:
            raise ValueError("Minimum singular value must be non-negative")
        
        if type(linear_operator) in [type([]),type(())] and len(linear_operator) == 3:
            U, D, VT = linear_operator
            self.factors = U, D, VT
        else:
            self.X = astransform(linear_operator)

    def copy(self):
        return factored_matrix([self.factors[0].copy(), self.factors[1].copy(), self.factors[2].copy()])

    def _setX(self,transform):
        transform = astransform(transform)
        self.input_shape = transform.input_shape
        self.output_shape = transform.output_shape
        U, D, VT = compute_iterative_svd(transform, 
                                         min_singular=self.min_singular, 
                                         tol=self.tol, 
                                         initial_rank=self.initial_rank, 
                                         warm_start=self.initial, 
                                         debug=self.debug)
        self.factors = [U,D,VT]

    def _getX(self):
        if not self.rankone:
            return np.dot(self.factors[0], np.dot(np.diag(self.factors[1]), self.factors[2]))
        else:
            return self.factors[1][0,0] * np.dot(self.factors[0], self.factors[2])
    X = property(_getX, _setX)

    def _get_factors(self):
        return self._factors
    def _set_factors(self, factors):
        U, D, VT = factors
        self.rankone = False
        if D.reshape(-1).shape == (1,):
            U = U.reshape((U.shape[0],1))
            D = D.reshape((1,1))
            VT = VT.reshape((1,VT.shape[-1]))
            self.rankone = True
        self.input_shape = (VT.shape[1],)
        self.output_shape = (U.shape[0],)
        self._factors = U, D, VT
    factors = property(_get_factors, _set_factors)

    def linear_map(self, x):
        if self.rankone:
            return self.factors[1][0,0] * np.dot(self.factors[0], np.dot(self.factors[2], x))
        else:
            return np.dot(self.factors[0], np.dot(np.diag(self.factors[1]), np.dot(self.factors[2], x)))

    def adjoint_map(self, x):
        if self.rankone:
            return self.factors[1][0,0] * np.dot(self.factors[2].T, np.dot(self.factors[0].T, x))
        else:
            return np.dot(self.factors[2].T, np.dot(np.diag(self.factors[1]), np.dot(self.factors[0].T, x)))

    def affine_map(self,x):
        if self.affine_offset is None:
            return self.offset_map(self.linear_map(x))
        else:
            return self.offset_map(self.linear_map(x))

    def offset_map(self, x, copy=False):
        if self.affine_offset is not None:
            return x + self.affine_offset
        else:
            return x


def compute_iterative_svd(transform,
                          initial_rank=None,
                          warm_start=None,
                          min_singular=1e-16,
                          tol=1e-5,
                          debug=False,
                          stopping_rule=None,
                          padding=5,
                          update_rank=lambda R: 2*R,
                          max_rank=None):

    """
    Compute the SVD of a matrix using partial_svd.

    If no initial rank is given, it assumes a rank of size min(n,p) / 10.

    Iteratively calls :func:`partial_svd` until the `singular_values` are small
    enough.

    Parameters
    ----------
    transform : [linear_transform, ndarray]
        Linear_transform whose SVD is computed. If an
        ndarray, it is first cast with :func:`astransform()`
    initial_rank : None or int, optional
        A guess at the rank of the matrix.
    warm_start : np.ndarray(np.float), optional
        A guess at the left singular vectors of the matrix.
        For fat matrices, these should be right singular vectors,
        while for tall matrices these should be left singular vectors.
    min_singular : np.float, optional
        Stop when the singular value has this relative tolerance.
    tol: np.float, optional
        Tolerance at which the norm of the singular values are deemed
        to have converged.
    debug: bool, optional
        Print debugging statements.
    stopping_rule : None or callable, optional
        Function of the singular values ``D``, returning True | False,  used to
        determine whether to stop.  Continue while ``stopping_rule(D) ==
        False``, or when `stopping_rule` is None.
    padding : int, optional
        How many more singular vectors are found.
    update_rank : callable, optional
        A rule to update rank, defaults to doubling the rank.
    max_rank : None or int, optional
        Largest rank considered. Defaults to 2 * min(transform.output_shape[0], 
        transform.input_shape[0]). An exception is raised if algorithm exceeds
        given value.

    Returns
    -------
    U, D, VT, Ufull : np.ndarray(np.float)
        An SVD of the transform. Ufull is the full set of left singular vectors
        found.

    Examples
    --------

    >>> import regreg.api as rr
    >>> np.random.seed(0)
    >>> X = np.random.standard_normal((100, 200))
    >>> U, D, VT = rr.compute_iterative_svd(X)[:3]
    >>> np.linalg.norm(np.dot(U.T, U) - np.identity(100)) < 1.e-6
    True

    >>> np.linalg.norm(np.dot(VT, VT.T) - np.identity(100)) < 1.e-6
    True
    """

    transform = astransform(transform)

    n = transform.output_shape[0]
    p = transform.input_shape[0]
    
    if max_rank is None:
        max_rank = 2 * min(n, p)

    need_to_transpose = False
    if n < p:
        transform = adjoint(transform)
        need_to_transpose = True

    if initial_rank is None:
        rank = np.round(np.min([n,p]) * 0.1) + 1
    else:
        rank = np.max([initial_rank,1])

    # for warm start
    if warm_start is not None:
        U = warm_start
    else:
        U = None

    min_so_far = 1.
    D = [min_so_far]
    while D[-1] >= min_singular * np.max(D):
        if debug:
            print("Trying rank", rank)
        U, D, VT = partial_svd(transform, rank=rank, 
                               padding=padding, tol=tol, 
                               warm_start=U,
                               return_full=True, debug=debug)
        if debug:
            print("Singular values", D)
        if len(D) < rank:
            break

        if stopping_rule is not None and stopping_rule(np.fabs(D)):
            break

        rank = update_rank(rank)
        if rank > max_rank:
            raise ValueError('maximum rank exceeded')

        if np.max(D) < 1.e-14:
            break

    ind = np.where(D >= min_singular)[0]
    if not need_to_transpose:
        if U.ndim == 2:
            return U[:,ind], D[ind],  VT[ind,:]
        else:
            return U.reshape((-1,1)), D[ind], VT.reshape((1,-1))
    else:
        if U.ndim == 2:
            return VT[ind,:].T, D[ind],  U[:,ind].T
        else:
            return VT.reshape((-1,1)), D[ind], U.reshape((1,-1))


def partial_svd(transform,
                rank=1,
                padding=5,
                max_its = 1000,
                tol = 1e-8,
                warm_start=None,
                return_full=False,
                debug=False,
                stopping_rule=None):

    """
    Compute partial SVD of the linear transform `transform`

    Uses the Mazumder/Hastie algorithm in (TODO: CITE)

    Parameters
    ----------
    transform : [linear_transform, ndarray]
        Linear_transform whose SVD is computed. If an
        ndarray, it is first cast with :func:`astransform()`

    rank : int, optional
        What rank of SVD to compute.

    padding : int, optional
        Compute a few further singular values / vectors. This results
        in a better estimator of the rank of interest.

    max_its : int, optional
        How many iterations of the full cycle to complete.

    tol : np.float, optional
        Tolerance at which the norm of the singular values are deemed
        to have converged.

    warm_start : np.ndarray(np.float), optional
        A guess at the singular vectors of the matrix. 
        For fat matrices, these should be right singular vectors,
        while for tall matrices these should be left singular vectors.

    return_full: bool, optional
        Return a singular values / vectors from padding?

    debug: bool, optional
        Print debugging statements.

    stopping_rule : callable
        Function of the singular values used to determine whether to stop.

    Returns
    -------
    U, D, VT, Ufull : np.ndarray(np.float)
        An SVD up to `rank` of the transform.
        Ufull is the full set of left singular vectors found.

    Examples
    --------

    >>> from regreg.affine.factored_matrix import partial_svd
    >>> np.random.seed(0)
    >>> X = np.random.standard_normal((100, 200))
    >>> U, D, VT = partial_svd(X, rank=10)[:3]
    >>> assert(np.linalg.norm(np.dot(U.T, U) - np.identity(10)) < 1.e-4)
    >>> assert(np.linalg.norm(np.dot(VT, VT.T) - np.identity(10)) < 1.e-4)
    >>> U_np, D_np, VT_np = np.linalg.svd(X, full_matrices=False)
    >>> assert(np.linalg.norm(U - np.dot(U, np.dot(U_np.T[:10], U_np[:,:10]))) < 1.e-4)
    >>> assert(np.linalg.norm(VT - np.dot(VT, np.dot(VT_np[:10].T, VT_np[:10]))) < 1.e-4)

    """

    transform = astransform(transform)

    n = np.product(transform.output_shape)
    p = np.product(transform.input_shape)

    rank = np.int(np.min([rank,p]))
    q = np.min([rank + padding, p])
    if warm_start is not None:
        if warm_start.shape == (n,q):
            U = warm_start
        elif len(warm_start.shape) == 1:
            U = np.hstack([warm_start.reshape((warm_start.shape[0],1)), np.random.standard_normal((n,q-1))])            
        else:
            U = np.hstack([warm_start, np.random.standard_normal((n,q-warm_start.shape[1]))])            
    else:
        U = np.random.standard_normal((n,q))

    if return_full:
        ind = np.arange(q)
    else:
        ind = np.arange(rank)

    old_singular_values = np.zeros(rank)
    change_ind = np.arange(rank)

    itercount = 0
    singular_rel_change = 1.

    while itercount < max_its and singular_rel_change > tol:
        if debug and itercount > 0:
            print(itercount, singular_rel_change,
                  np.sum(np.fabs(singular_values)>1e-12),
                  np.fabs(singular_values[np.arange(np.min([5,len(singular_values)]))]))
        V, _ = np.linalg.qr(transform.adjoint_map(U))
        X_V = transform.linear_map(V)
        U, R = np.linalg.qr(X_V)
        singular_values = np.diagonal(R)[change_ind]
        singular_rel_change = np.linalg.norm(singular_values - old_singular_values)/np.linalg.norm(singular_values)
        old_singular_values[:] = singular_values
        itercount += 1

        if stopping_rule is not None and stopping_rule(np.fabs(singular_values)):
            break

    singular_values = np.diagonal(R)[ind]

    nonzero = np.where(np.fabs(singular_values) > 1e-12)[0]
    if len(nonzero):
        return U[:,ind[nonzero]] * np.sign(singular_values[nonzero]), np.fabs(singular_values[nonzero]),  V[:,ind[nonzero]].T
    else:
        return U[:,ind[0]], np.zeros((1,1)),  V[:,ind[0]].T

@objective_doc_templater()
class nuclear_norm(nuclear_norm_atom):

    """
    The nuclear norm
    """
    prox_tol = 1.0e-5
    svd_tol = 1.e-5
    objective_vars = nuclear_norm_atom.objective_vars.copy()

    def __init__(self, shape, lagrange=None, bound=None,
                 offset=None, quadratic=None, initial=None,
                 initial_rank=10,
                 warm_start=None):

        nuclear_norm_atom.__init__(self,
                                   shape,
                                   lagrange=lagrange,
                                   bound=bound,
                                   offset=offset,
                                   quadratic=quadratic,
                                   initial=0)

        self.initial_rank = initial_rank
        # the warm start should be of whichever side has largest
        # dimension
        m = max(self.shape)
        if warm_start is None:
            self.warm_start = np.random.standard_normal((m, self.initial_rank))
        else:
            self.warm_start = warm_start
            if warm_start.shape != (m, self.initial_rank):
                raise ValueError('expecting warm_start to have shape %s' % (m, self.initial_rank))

    @doc_template_user
    def seminorm(self, X, check_feasibility=False,
                 lagrange=None):
        raise NotImplementedError('too expensive to compute')

    @doc_template_user
    def constraint(self, X, bound=None):
        raise NotImplementedError('too expensive to compute')

    @doc_template_user
    def lagrange_prox(self, X,  lipschitz=1, lagrange=None):
        lagrange = svd_atom.lagrange_prox(self, X, lipschitz, lagrange)

        def soft_threshold_rule(L):
            return lambda D: np.fabs(D).min() <= L

        svt_rule = soft_threshold_rule(lagrange)
        if self.warm_start is not None:
            padding = 0
        else:
            padding = 5

        U, D, VT = compute_iterative_svd(X, initial_rank=self.initial_rank, 
                                         warm_start=self.warm_start,
                                         stopping_rule=svt_rule, 
                                         tol=self.svd_tol,
                                         padding=padding)

        if self.shape[0] < self.shape[1]:
            self.warm_start = VT.T
        else:
            self.warm_start = U

        self.initial_rank = U.shape[1]

        D_proj = D - lagrange
        keep = D_proj > 0

        return factored_matrix((U[:,keep], D_proj[keep], VT[keep,:]))

    @doc_template_user
    def bound_prox(self, X, bound=None):
        bound = svd_atom.bound_prox(self, X, bound)

        def bound_rule(B):
            return lambda D: (np.fabs(D) - np.fabs(D).min()).sum() > B

        if self.warm_start is not None:
            padding = 0
        else:
            padding = 5

        U, D, VT = compute_iterative_svd(X, initial_rank=self.initial_rank,
                                         warm_start=self.warm_start,
                                         stopping_rule=bound_rule(bound),
                                         tol=self.svd_tol,
                                         padding=padding)

        if self.shape[0] < self.shape[1]:
            self.warm_start = VT.T
        else:
            self.warm_start = U

        self.initial_rank = U.shape[1]

        l1atom = l1norm(D.shape, bound=bound)
        D_projected = l1atom.bound_prox(D)
        keep = D_projected > 0
        return factored_matrix((U[:,keep], D_projected[keep], VT[keep,:]))

    @property
    def conjugate(self):
        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)

            cls = operator_norm
            atom = cls(self.shape,
                       bound=self.lagrange, 
                       lagrange=self.bound,
                       quadratic=outq,
                       offset=offset,
                       initial_rank=self.initial_rank,
                       warm_start=self.warm_start)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate



@objective_doc_templater()
class operator_norm(operator_norm_atom):
    prox_tol = 1.0e-5
    svd_tol = 1.e-4
    objective_vars = operator_norm_atom.objective_vars.copy()

    def __init__(self, shape, lagrange=None, bound=None,
                 offset=None, quadratic=None, initial=None,
                 initial_rank=10, warm_start=None):

        operator_norm_atom.__init__(self,
                                    shape,
                                    lagrange=lagrange,
                                    bound=bound,
                                    offset=offset,
                                    quadratic=quadratic,
                                    initial=0)

        self.initial_rank = initial_rank
        if warm_start is None:
            self.warm_start = np.random.standard_normal((self.shape[0], self.initial_rank))
        else:
            self.warm_start = warm_start
            if warm_start.shape != (self.shape[0], self.initial_rank):
                raise ValueError('expecting warm_start to have shape %s' % (self.shape[0], self.initial_rank))

        self._nuclear_atom = nuclear_norm(shape,
                                          lagrange=lagrange,
                                          bound=bound,
                                          initial=initial,
                                          initial_rank=initial_rank,
                                          warm_start=self.warm_start)

    @doc_template_user
    def seminorm(self, X, check_feasibility=False,
                 lagrange=None):
        raise NotImplementedError('too expensive to compute')

    @doc_template_user
    def constraint(self, X, bound=None):
        raise NotImplementedError('too expensive to compute')

    @doc_template_user
    def lagrange_prox(self, X,  lipschitz=1, lagrange=None):
        lagrange = svd_atom.lagrange_prox(self, X, lipschitz, lagrange)

        bound = lagrange / lipschitz
        dual_proj = self._nuclear_atom.bound_prox(X, bound=bound)
        return affine_sum([X,dual_proj],[1.,-1.])

    @doc_template_user
    def bound_prox(self, X,  bound=None):
        bound = svd_atom.bound_prox(self, X, bound)

        lagrange = bound
        dual_lagrange = self._nuclear_atom.lagrange_prox(X, lagrange=lagrange)
        return affine_sum([X,dual_lagrange],[1.,-1.])

    @property
    @doc_template_user
    def conjugate(self):
        if self.quadratic.coef == 0:
            offset, outq = _work_out_conjugate(self.offset, 
                                               self.quadratic)

            cls = nuclear_norm
            atom = cls(self.shape,
                       bound=self.lagrange, 
                       lagrange=self.bound,
                       quadratic=outq,
                       offset=offset,
                       initial_rank=self.initial_rank,
                       warm_start=self.warm_start)
        else:
            atom = smooth_conjugate(self)
        self._conjugate = atom
        self._conjugate._conjugate = self
        return self._conjugate



import numpy as np

from .. import lasso, sparse_group_lasso, strong_rules, warm_start
from ..basil import basil_inner_loop, basil
from ...tests.decorators import set_seed_for_test

@set_seed_for_test()
def test_lasso_agreement1(n=200,p=50):
    '''
    check to see if it agrees with lasso path

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = np.arange(p)
    l1weights = np.ones(p)
    weights = dict([(j,0) for j in np.arange(p)])
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      l1_alpha=1,
                                                      weights=weights)
    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_lasso1, 
                        lagrange_sequence, 
                        (sparse_group_lasso1.solution, sparse_group_lasso1.grad_solution),
                        inner_tol=1.e-12)

    weights = np.ones(p)

    lasso2 = lasso.gaussian(X, 
                            Y, 
                            weights)

    lagrange_sequence2 = lasso.default_lagrange_sequence(lasso2.penalty,
                                                         lasso2.grad_solution,
                                                         nstep=23) # initialized at "null" model
    sol2 = strong_rules(lasso2, 
                        lagrange_sequence, 
                        (lasso2.solution, lasso2.grad_solution),
                        inner_tol=1.e-15)
    beta1 = sol1['beta']
    beta2 = sol2['beta']

    assert(np.linalg.norm(beta1 - beta2) / np.linalg.norm(beta1) < 1.e-5)

@set_seed_for_test()
def test_path_subsample(n=200,p=50):
    '''
    compare a subsample path to the full path on subsampled data

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = ['a', 'a', 'a', 'b', 'b']
    i = 0
    while True:
        groups.extend([str(i)]*5)
        i += 1
        if len(groups) > p:
            break
    groups = groups[:p]

    np.random.shuffle(groups)
    l1weights = np.ones(p)
    weights = {}
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      l1_alpha=1,
                                                      weights=weights)
    cases = range(100)
    sparse_group_lasso1 = sparse_group_lasso1.subsample(cases)
    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_lasso1, 
                        lagrange_sequence, 
                        (sparse_group_lasso1.solution, sparse_group_lasso1.grad_solution), 
                        inner_tol=1.e-10)
    beta1 = sol1['beta']

    sparse_group_lasso2 = sparse_group_lasso.gaussian(X[cases], 
                                                      Y[cases], 
                                                      groups,
                                                      l1weights,
                                                      l1_alpha=1,
                                                      weights=weights)
    lagrange_sequence2 = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso2.penalty,
                                                                      sparse_group_lasso2.grad_solution,
                                                                      nstep=23) # initialized at "null" model

    sol2 = strong_rules(sparse_group_lasso2, 
                        lagrange_sequence, 
                        (sparse_group_lasso2.solution, sparse_group_lasso2.grad_solution),
                        inner_tol=1.e-10)
    beta2 = sol2['beta']

    np.testing.assert_allclose(beta1, beta2, rtol=1.e-3)

@set_seed_for_test()
def test_lasso_agreement2(n=200,p=50):
    '''
    check to see if it agrees with lasso path

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = np.arange(p)
    l1weights = np.zeros(p)
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      l1_alpha=0)
    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_lasso1, 
                        lagrange_sequence, 
                        (sparse_group_lasso1.solution, sparse_group_lasso1.grad_solution),
                        inner_tol=1.e-12)

    weights = np.ones(p)
    lasso2 = lasso.gaussian(X, 
                            Y, 
                            weights)
    sol2 = strong_rules(lasso2, 
                        lagrange_sequence, 
                        (lasso2.solution, lasso2.grad_solution),
                        inner_tol=1.e-15)
    beta1 = sol1['beta']
    beta2 = sol2['beta']

    assert(np.linalg.norm(beta1 - beta2) / np.linalg.norm(beta1) < 1.e-5)

@set_seed_for_test()
def test_path():
    '''
    run a basic path algorithm

    '''
    n, p = 200, 50
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    np.random.shuffle(groups)
    l1weights = np.ones(p)
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights)
    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_lasso1, 
                        lagrange_sequence, 
                        (sparse_group_lasso1.solution, sparse_group_lasso1.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_unpenalized(n=200, p=50):
    '''
    run a basic path algorithm with some unpenalized variables

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['2'] = weights['3'] = 0
    weights['a'] = weights['b'] = 2
    l1weights = np.ones(p)
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      weights=weights)
    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_lasso1, 
                        lagrange_sequence, 
                        (sparse_group_lasso1.solution, sparse_group_lasso1.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_elastic_net(n=200, p=50):
    '''
    run a basic elastic net path algorithm

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    np.random.shuffle(betaX)
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.zeros(X.shape[1])
    enet[4:7] = 0

    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    l1weights = np.ones(p)
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      alpha=0.5,
                                                      elastic_net_param=enet)
    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_lasso1, 
                        lagrange_sequence, 
                        (sparse_group_lasso1.solution, sparse_group_lasso1.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']

@set_seed_for_test()
def test_elastic_net_unpenalized(n=200, p=50):
    '''
    run a basic elastic net path algorithm with unpenalized

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[4:8] = 0

    groups = ['a', 'a', 'a', 'b', 'b']
    for i in range(9):
        groups.extend([str(i)]*5)

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['2'] = weights['3'] = 0
    weights['a'] = weights['b'] = 2

    l1weights = np.ones(p)
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      weights=weights,
                                                      alpha=0.5,
                                                      elastic_net_param=enet)
    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=23) # initialized at "null" model
    sol1 = strong_rules(sparse_group_lasso1, 
                        lagrange_sequence, 
                        (sparse_group_lasso1.solution, sparse_group_lasso1.grad_solution),
                        inner_tol=1.e-12)
    beta1 = sol1['beta']


@set_seed_for_test()
def test_basil_inner_loop(n=600,p=200):
    '''
    test one run of the BASIL inner loop

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    enet = np.ones(X.shape[1])
    enet[4:8] = 0

    groups = ['a', 'a', 'a', 'b', 'b']
    i = 0
    while True:
        groups.extend([str(i)]*5)
        i += 1
        if len(groups) > p:
            break
    groups = groups[:p]

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['2'] = weights['3'] = 0
    weights['a'] = weights['b'] = 2

    l1weights = np.ones(p)
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      weights=weights,
                                                      alpha=0.5,
                                                      elastic_net_param=enet)
    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=100) # initialized at "null" model
    sol1 = basil_inner_loop(sparse_group_lasso1, 
                            lagrange_sequence[:50], 
                            (sparse_group_lasso1.solution.copy(), sparse_group_lasso1.grad_solution.copy()),
                            inner_tol=1.e-14,
                            step_nvar=10)
    lagrange1, beta1, grad1, active1 = sol1
    print(np.array(beta1).shape, 'chunk of path')

    print(active1, 'active')

@set_seed_for_test()
def test_basil(n=200,p=100):
    '''
    run BASIL

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = ['a', 'a', 'a', 'b', 'b']
    i = 0
    while True:
        groups.extend([str(i)]*5)
        i += 1
        if len(groups) > p:
            break
    groups = groups[:p]

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['a'] = weights['b'] = 2

    l1weights = np.ones(p)
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      weights=weights)

    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     lagrange_proportion=0.5,
                                                                     nstep=20) # initialized at "null" model
    print(sparse_group_lasso1.penalty.conjugate.seminorm(sparse_group_lasso1.grad_solution, lagrange=1))
    print(lagrange_sequence, 'lagrange 1')
    sol1 = basil(sparse_group_lasso1, 
                 lagrange_sequence.copy(), 
                 (sparse_group_lasso1.solution.copy(), sparse_group_lasso1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)
    
    print(lagrange_sequence, 'lagrange 2')

    sparse_group_lasso2 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      weights=weights)
    print(sparse_group_lasso2.penalty.conjugate.seminorm(sparse_group_lasso2.grad_solution, lagrange=1))
    print(lagrange_sequence, 'lagrange 3')

    sol2 = warm_start(sparse_group_lasso2, 
                      lagrange_sequence, 
                      (sparse_group_lasso2.solution.copy(), sparse_group_lasso2.grad_solution.copy()),
                      inner_tol=1.e-14)['beta']

    print(lagrange_sequence, 'lagrange 4')
    print(sparse_group_lasso2.penalty.conjugate.seminorm(sparse_group_lasso2.grad_solution, lagrange=1))
    print(sparse_group_lasso1.penalty.conjugate.seminorm(sparse_group_lasso1.grad_solution, lagrange=1))
    print(np.linalg.norm(sparse_group_lasso1.grad_solution - sparse_group_lasso2.grad_solution))
    assert(np.linalg.norm(sol1 - sol2) / np.linalg.norm(sol2) < 1.e-4)

@set_seed_for_test()
def test_basil_unpenalized(n=200,p=100):
    '''
    run BASIL w/ unpenalized groups

    '''
    X = np.random.standard_normal((n,p))
    Y = np.random.standard_normal(n)
    betaX = np.zeros(p)
    betaX[:3] = [3,4,5]
    Y += np.dot(X, betaX) + np.random.standard_normal(n)

    groups = ['a', 'a', 'a', 'b', 'b']
    i = 0
    while True:
        groups.extend([str(i)]*5)
        i += 1
        if len(groups) > p:
            break
    groups = groups[:p]

    weights = dict([(g,1) for g in np.unique(groups)])
    weights['a'] = weights['b'] = 2
    weights['2'] = weights['3'] = 0
    l1weights = np.ones(p)
    sparse_group_lasso1 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      weights=weights)

    lagrange_sequence = sparse_group_lasso.default_lagrange_sequence(sparse_group_lasso1.penalty,
                                                                     sparse_group_lasso1.grad_solution,
                                                                     nstep=100) # initialized at "null" model
    sol1 = basil(sparse_group_lasso1, 
                 lagrange_sequence, 
                 (sparse_group_lasso1.solution.copy(), sparse_group_lasso1.grad_solution.copy()),
                 inner_tol=1.e-14,
                 step_nvar=10,
                 step_lagrange=20)
    
    sparse_group_lasso2 = sparse_group_lasso.gaussian(X, 
                                                      Y, 
                                                      groups,
                                                      l1weights,
                                                      weights=weights)
    sol2 = warm_start(sparse_group_lasso2, 
                      lagrange_sequence, 
                      (sparse_group_lasso2.solution.copy(), sparse_group_lasso2.grad_solution.copy()),
                      inner_tol=1.e-14)['beta']

    assert(np.linalg.norm(sol1 - sol2) / np.linalg.norm(sol2) < 1.e-4)
